import xmltodict

from .. helperFuncs import make_request as mr

class ITIS:
    """ 
    A class for interacting with the Integrated Taxonomic Integration System API.
    
    Attributes:
    - base_url: The base URL of the API.
    - about: A short description of the API.
    """
    def __init__(self):
        self.base_url = "http://www.itis.gov/ITISWebService/services/ITISService/"
        self.about = "The ITIS program is driven by a mission: communicate a comprehensive taxonomy of global species that enables biodiversity information to be discovered, indexed, and connected across all human endeavors."
        
    def get_docs_url(self):
        """
        Returns the URL for the ITIS API documentation.
        
        Args:
        - None
        
        Returns:
        - string: The URL for the API docs.
        """
        return "https://www.itis.gov/ws_description.html"
        
    #SEARCH FUNCTIONS
    def search_for_any_match(self, search_key, data_format="json"):
        """
        Returns matches found by comparing the search key to the ITIS common names, scientific names, and TSNs.
        
        Args:
        - search_key (str or int): The name (common or scientific) or Taxonomic Serial Number (TSN).
        - data_format (str): The desired format for returned data. Defaults to json, but other option is xml.
        
        Returns:
        - dict or string (xml): A dictionary or XML string containing the data matching the search key.
        """
        endpoint = f"searchForAnyMatch?srchKey={search_key}"
        data_format = data_format.lower()
        if data_format == "json":
            data = mr.make_request_for_content(self.base_url+endpoint)
            return xmltodict.parse(data)
        elif data_format == "xml":
            return mr.make_request_for_content(self.base_url+endpoint)
        else:
            return "Error: Format must be either json or xml."
            
    def search_for_any_match_paged(self, search_key, page_size=50, page_num=1, ascend=True, data_format="json"):
        """
        Returns matches found by comparing the search key to the ITIS common names, scientific names, and TSNs.
        
        Args:
        - search_key (str or int): The name (common or scientific) or Taxonomic Serial Number (TSN).
        - page_size (int): Number of matches returned per page. Defaults to 50.
        - page_num (int): The page number to return (pagination). Defaults to the first page.
        - ascend (boolean): Whether to sort pages in ascending or descending alphabetical order. Default is True.
        - data_format (str): The desired format for returned data. Defaults to json, but other option is xml.
           
        Returns:
        - dict or string (xml): A dictionary or XML string containing the data matching the search key.
        """
        endpoint = f"searchForAnyMatchPaged?srchKey={search_key}&pageSize={page_size}&pageNum={page_num}&ascend={ascend}"
        data_format = data_format.lower()
        if data_format == "json":
            data = mr.make_request_for_content(self.base_url+endpoint)
            return xmltodict.parse(data)
        elif data_format == "xml":
            return mr.make_request_for_content(self.base_url+endpoint)
        else:
            return "Error: Format must be either json or xml."
            
    def get_any_match_count(self, search_key, data_format="json"):
        """
        Returns a count of the matches found by comparing the search key to the ITIS common names, scientific names, and TSNs..
        
        Args:
        - search_key (str or int): The name (common or scientific) or Taxonomic Serial Number (TSN).
        - data_format (str): The desired format for returned data. Defaults to json, but other option is xml.
        
        Returns:
        - dict or string (xml): A dictionary or XML string containing the count matching the search key.
        """
        endpoint = f"getAnyMatchCount?srchKey={search_key}"
        data_format = data_format.lower()
        if data_format == "json":
            data = mr.make_request_for_content(self.base_url+endpoint)
            return xmltodict.parse(data)
        elif data_format == "xml":
            return mr.make_request_for_content(self.base_url+endpoint)
        else:
            return "Error: Format must be either json or xml."
            
    def search_by_common_name(self, search_key, data_format="json"):
        """
        Returns matches found by comparing the search key to the ITIS common names.
        
        Args:
        - search_key (str): The common name.
        - data_format (str): The desired format for returned data. Defaults to json, but other option is xml.
        
        Returns:
        - dict or string (xml): A dictionary or XML string containing the data matching the search key.
        """
        endpoint = f"searchByCommonName?srchKey={search_key}"
        data_format = data_format.lower()
        if data_format == "json":
            data = mr.make_request_for_content(self.base_url+endpoint)
            return xmltodict.parse(data)
        elif data_format == "xml":
            return mr.make_request_for_content(self.base_url+endpoint)
        else:
            return "Error: Format must be either json or xml."
            
    def search_by_common_name_begins_with(self, search_key, data_format="json"):
        """
        Return matches found by comparing the search key to the beginning of the ITIS common names.
        
        Args:
        - search_key (str): The beginning of the name to be searched.
        - data_format (str): The desired format for returned data. Defaults to json, but other option is xml.
        
        Returns:
        - dict or string (xml): A dictionary or XML string containing the data matching the search key.
        """
        endpoint = f"searchByCommonNameBeginsWith?srchKey={search_key}"
        data_format = data_format.lower()
        if data_format == "json":
            data = mr.make_request_for_content(self.base_url+endpoint)
            return xmltodict.parse(data)
        elif data_format == "xml":
            return mr.make_request_for_content(self.base_url+endpoint)
        else:
            return "Error: Format must be either json or xml."
            
    def search_by_common_name_ends_with(self, search_key, data_format="json"):
        """
        Return matches found by comparing the search key to the end of the ITIS common names.
        
        Args:
        - search_key (str): The end of the name to be searched.
        - data_format (str): The desired format for returned data. Defaults to json, but other option is xml.
        
        Returns:
        - dict or string (xml): A dictionary or XML string containing the data matching the search key.
        """
        endpoint = f"searchByCommonNameEndsWith?srchKey={search_key}"
        data_format = data_format.lower()
        if data_format == "json":
            data = mr.make_request_for_content(self.base_url+endpoint)
            return xmltodict.parse(data)
        elif data_format == "xml":
            return mr.make_request_for_content(self.base_url+endpoint)
        else:
            return "Error: Format must be either json or xml."
            
    def search_by_scientific_name(self, search_key, data_format="json"):
        """
        Returns matches found by comparing the search key to the ITIS scientific names.
        
        Args:
        - search_key (str): The scientific name.
        - data_format (str): The desired format for returned data. Defaults to json, but other option is xml.
        
        Returns:
        - dict or string (xml): A dictionary or XML string containing the data matching the search key.
        """
        endpoint = f"searchByScientificName?srchKey={search_key}"
        data_format = data_format.lower()
        if data_format == "json":
            data = mr.make_request_for_content(self.base_url+endpoint)
            return xmltodict.parse(data)
        elif data_format == "xml":
            return mr.make_request_for_content(self.base_url+endpoint)
        else:
            return "Error: Format must be either json or xml."
            
    def get_ITIS_terms(self, search_key, data_format="json"):
        """
        Gets a list of ITIS Terms as used by the Resource Catalog Input Tool from a common or scientific name match, or an empty result set if there is no match.
        
        Args:
        - search_key (str): The common or scientific name.
        - data_format (str): The desired format for returned data. Defaults to json, but other option is xml.
        
        Returns:
        - dict or string (xml): A dictionary or XML string containing the data matching the search key.
        """
        endpoint = f"getITISTerms?srchKey={search_key}"
        data_format = data_format.lower()
        if data_format == "json":
            data = mr.make_request_for_content(self.base_url+endpoint)
            return xmltodict.parse(data)
        elif data_format == "xml":
            return mr.make_request_for_content(self.base_url+endpoint)
        else:
            return "Error: Format must be either json or xml."
            
    def get_ITIS_terms_from_common_name(self, search_key, data_format="json"):
        """
        Gets a list of ITIS Terms as used by the Resource Catalog Input Tool from a common name match, or an empty result set if there is no match.
        
        Args:
        - search_key (str): The common name.
        - data_format (str): The desired format for returned data. Defaults to json, but other option is xml.
        
        Returns:
        - dict or string (xml): A dictionary or XML string containing the data matching the search key.
        """
        endpoint = f"getITISTermsFromCommonName?srchKey={search_key}"
        data_format = data_format.lower()
        if data_format == "json":
            data = mr.make_request_for_content(self.base_url+endpoint)
            return xmltodict.parse(data)
        elif data_format == "xml":
            return mr.make_request_for_content(self.base_url+endpoint)
        else:
            return "Error: Format must be either json or xml."
            
    def get_ITIS_terms_from_scientific_name(self, search_key, data_format="json"):
        """
        Gets a list of ITIS Terms as used by the Resource Catalog Input Tool from a scientific name match, or an empty result set if there is no match.
        
        Args:
        - search_key (str): The scientific name.
        - data_format (str): The desired format for returned data. Defaults to json, but other option is xml.
        
        Returns:
        - dict or string (xml): A dictionary or XML string containing the data matching the search key.
        """
        endpoint = f"getITISTermsFromScientificName?srchKey={search_key}"
        data_format = data_format.lower()
        if data_format == "json":
            data = mr.make_request_for_content(self.base_url+endpoint)
            return xmltodict.parse(data)
        elif data_format == "xml":
            return mr.make_request_for_content(self.base_url+endpoint)
        else:
            return "Error: Format must be either json or xml."
            
    def get_TSNs_by_vernacular_language(self, language, data_format="json"):
        """
        Provide a listing of Taxonomic Serial Numbers (TSNs) with vernaculars entered in the requested language.
        
        Args:
        - language (str): The desired language. Supported values are Afrikaans, Arabic, Australian, Bengali, Chinese, Djuka, Dutch, eng, English, Fijan, French, Galibi, German, Greek, Hausa, Hawaiian, Hindi, Icelandic, Iglulik Inuit, Italian, Japanese, Javanese, Khmer, Korean, Lao, Malagasy, Manyika, Native American, Ndau, Northern Sotho, Portuguese, Romanian, Shona, Spanish, Swati, unspecified, Xhosa, Zulu.
        - data_format (str): The desired format for returned data. Defaults to json, but other option is xml.
        
        Returns:
        - dict or string (xml): A dictionary or XML string containing the data matching the language.
        """
        endpoint = f"getTsnByVernacularLanguage?language={language}"
        data_format = data_format.lower()
        if data_format == "json":
            data = mr.make_request_for_content(self.base_url+endpoint)
            return xmltodict.parse(data)
        elif data_format == "xml":
            return mr.make_request_for_content(self.base_url+endpoint)
        else:
            return "Error: Format must be either json or xml."
            
         
    #FUNCTIONS TO RETRIEVE INFORMATION BY TSN
    def get_accepted_names_from_TSN(self, tsn, data_format="json"):
        """
        Returns a list of accepted names for the TSN. This can return more than one result if the TSN is for a synonym.
        
        Args:
        - tsn (int): The Taxonomic Serial Number (TSN).
        - data_format (str): The desired format for returned data. Defaults to json, but other option is xml.
        
        Returns:
        - dict or string (xml): A dictionary or XML string containing the data matching the TSN.
        """
        endpoint = f"getAcceptedNamesFromTSN?tsn={tsn}"
        data_format = data_format.lower()
        if data_format == "json":
            data = mr.make_request_for_content(self.base_url+endpoint)
            return xmltodict.parse(data)
        elif data_format == "xml":
            return mr.make_request_for_content(self.base_url+endpoint)
        else:
            return "Error: Format must be either json or xml."
            
    def get_comment_detail_from_TSN(self, tsn, data_format="json"):
        """
        Returns the list of comments for the TSN. This can return more than result if multiple comments are entered.
        
        Args:
        - tsn (int): The Taxonomic Serial Number (TSN).
        - data_format (str): The desired format for returned data. Defaults to json, but other option is xml.
        
        Returns:
        - dict or string (xml): A dictionary or XML string containing the data matching the TSN.
        """
        endpoint = f"getCommentDetailFromTSN?tsn={tsn}"
        data_format = data_format.lower()
        if data_format == "json":
            data = mr.make_request_for_content(self.base_url+endpoint)
            return xmltodict.parse(data)
        elif data_format == "xml":
            return mr.make_request_for_content(self.base_url+endpoint)
        else:
            return "Error: Format must be either json or xml."
            
    def get_common_names_from_TSN(self, tsn, data_format="json"):
        """
        Returns a list of common names for the TSN.
        
        Args:
        - tsn (int): The Taxonomic Serial Number (TSN).
        - data_format (str): The desired format for returned data. Defaults to json, but other option is xml.
        
        Returns:
        - dict or string (xml): A dictionary or XML string containing the data matching the TSN.
        """
        endpoint = f"getCommonNamesFromTSN?tsn={tsn}"
        data_format = data_format.lower()
        if data_format == "json":
            data = mr.make_request_for_content(self.base_url+endpoint)
            return xmltodict.parse(data)
        elif data_format == "xml":
            return mr.make_request_for_content(self.base_url+endpoint)
        else:
            return "Error: Format must be either json or xml."
            
    def get_core_metadata_from_TSN(self, tsn, data_format="json"):
        """
        Returns the set of core metadata for the TSN. This contains credibility rating, taxonomic usage, unacceptability reason, coverage, and currency data.
        
        Args:
        - tsn (int): The Taxonomic Serial Number (TSN).
        - data_format (str): The desired format for returned data. Defaults to json, but other option is xml.
        
        Returns:
        - dict or string (xml): A dictionary or XML string containing the data matching the TSN.
        """
        endpoint = f"getCoreMetadataFromTSN?tsn={tsn}"
        data_format = data_format.lower()
        if data_format == "json":
            data = mr.make_request_for_content(self.base_url+endpoint)
            return xmltodict.parse(data)
        elif data_format == "xml":
            return mr.make_request_for_content(self.base_url+endpoint)
        else:
            return "Error: Format must be either json or xml."
            
    def get_coverage_from_TSN(self, tsn, data_format="json"):
        """
        Returns the taxon coverage information for the TSN. This information is  available for Genus and above (rank_id > 190) only.
        
        Args:
        - tsn (int): The Taxonomic Serial Number (TSN).
        - data_format (str): The desired format for returned data. Defaults to json, but other option is xml.
        
        Returns:
        - dict or string (xml): A dictionary or XML string containing the data matching the TSN.
        """
        endpoint = f"getCoverageFromTSN?tsn={tsn}"
        data_format = data_format.lower()
        if data_format == "json":
            data = mr.make_request_for_content(self.base_url+endpoint)
            return xmltodict.parse(data)
        elif data_format == "xml":
            return mr.make_request_for_content(self.base_url+endpoint)
        else:
            return "Error: Format must be either json or xml."
            
    def get_credibility_rating_from_TSN(self, tsn, data_format="json"):
        """
        Returns the credibility rating for the TSN.
        
        Args:
        - tsn (int): The Taxonomic Serial Number (TSN).
        - data_format (str): The desired format for returned data. Defaults to json, but other option is xml.
        
        Returns:
        - dict or string (xml): A dictionary or XML string containing the data matching the TSN.
        """
        endpoint = f"getCredibilityRatingFromTSN?tsn={tsn}"
        data_format = data_format.lower()
        if data_format == "json":
            data = mr.make_request_for_content(self.base_url+endpoint)
            return xmltodict.parse(data)
        elif data_format == "xml":
            return mr.make_request_for_content(self.base_url+endpoint)
        else:
            return "Error: Format must be either json or xml."
            
    def get_currency_from_TSN(self, tsn, data_format="json"):
        """
        Returns the taxon currency information for the TSN. This information is  available for Genus and above (rank_id > 190) only.
        
        Args:
        - tsn (int): The Taxonomic Serial Number (TSN).
        - data_format (str): The desired format for returned data. Defaults to json, but other option is xml.
        
        Returns:
        - dict or string (xml): A dictionary or XML string containing the data matching the TSN.
        """
        endpoint = f"getCurrencyFromTSN?tsn={tsn}"
        data_format = data_format.lower()
        if data_format == "json":
            data = mr.make_request_for_content(self.base_url+endpoint)
            return xmltodict.parse(data)
        elif data_format == "xml":
            return mr.make_request_for_content(self.base_url+endpoint)
        else:
            return "Error: Format must be either json or xml."
            
    def get_date_data_from_TSN(self, tsn, data_format="json"):
        """
        Returns the initial time stamp and last update date for the TSN.
        
        Args:
        - tsn (int): The Taxonomic Serial Number (TSN).
        - data_format (str): The desired format for returned data. Defaults to json, but other option is xml.
        
        Returns:
        - dict or string (xml): A dictionary or XML string containing the data matching the TSN.
        """
        endpoint = f"getDateDataFromTSN?tsn={tsn}"
        data_format = data_format.lower()
        if data_format == "json":
            data = mr.make_request_for_content(self.base_url+endpoint)
            return xmltodict.parse(data)
        elif data_format == "xml":
            return mr.make_request_for_content(self.base_url+endpoint)
        else:
            return "Error: Format must be either json or xml."
            
    def get_experts_from_TSN(self, tsn, data_format="json"):
        """
        Returns a list of the expert information for the TSN.
        
        Args:
        - tsn (int): The Taxonomic Serial Number (TSN).
        - data_format (str): The desired format for returned data. Defaults to json, but other option is xml.
        
        Returns:
        - dict or string (xml): A dictionary or XML string containing the data matching the TSN.
        """
        endpoint = f"getExpertsFromTSN?tsn={tsn}"
        data_format = data_format.lower()
        if data_format == "json":
            data = mr.make_request_for_content(self.base_url+endpoint)
            return xmltodict.parse(data)
        elif data_format == "xml":
            return mr.make_request_for_content(self.base_url+endpoint)
        else:
            return "Error: Format must be either json or xml."
            
    def get_full_record_from_TSN(self, tsn, data_format="json"):
        """
        Returns the full ITIS record for a TSN found by comparing the search key to the TSN field, or an empty result set if there is no match.
        
        Args:
        - tsn (int): The Taxonomic Serial Number (TSN).
        - data_format (str): The desired format for returned data. Defaults to json, but other option is xml.
        
        Returns:
        - dict or string (xml): A dictionary or XML string containing the data matching the TSN.
        """
        endpoint = f"getFullRecordFromTSN?tsn={tsn}"
        data_format = data_format.lower()
        if data_format == "json":
            data = mr.make_request_for_content(self.base_url+endpoint)
            return xmltodict.parse(data)
        elif data_format == "xml":
            return mr.make_request_for_content(self.base_url+endpoint)
        else:
            return "Error: Format must be either json or xml."
            
    def get_geographic_divisions_from_TSN(self, tsn, data_format="json"):
        """
        Returns a list of the geographic divisions for the TSN.
        
        Args:
        - tsn (int): The Taxonomic Serial Number (TSN).
        - data_format (str): The desired format for returned data. Defaults to json, but other option is xml.
        
        Returns:
        - dict or string (xml): A dictionary or XML string containing the data matching the TSN.
        """
        endpoint = f"getGeographicDivisionsFromTSN?tsn={tsn}"
        data_format = data_format.lower()
        if data_format == "json":
            data = mr.make_request_for_content(self.base_url+endpoint)
            return xmltodict.parse(data)
        elif data_format == "xml":
            return mr.make_request_for_content(self.base_url+endpoint)
        else:
            return "Error: Format must be either json or xml."
            
    def get_global_species_completeness_from_TSN(self, tsn, data_format="json"):
        """
        Returns the taxon completenes rating for the TSN.
        
        Args:
        - tsn (int): The Taxonomic Serial Number (TSN).
        - data_format (str): The desired format for returned data. Defaults to json, but other option is xml.
        
        Returns:
        - dict or string (xml): A dictionary or XML string containing the data matching the TSN.
        """
        endpoint = f"getGlobalSpeciesCompletenessFromTSN?tsn={tsn}"
        data_format = data_format.lower()
        if data_format == "json":
            data = mr.make_request_for_content(self.base_url+endpoint)
            return xmltodict.parse(data)
        elif data_format == "xml":
            return mr.make_request_for_content(self.base_url+endpoint)
        else:
            return "Error: Format must be either json or xml."
            
    def get_jurisdictional_origin_from_TSN(self, tsn, data_format="json"):
        """
        Returns a list of the jurisdiction and origin values for the TSN.
        
        Args:
        - tsn (int): The Taxonomic Serial Number (TSN).
        - data_format (str): The desired format for returned data. Defaults to json, but other option is xml.
        
        Returns:
        - dict or string (xml): A dictionary or XML string containing the data matching the TSN.
        """
        endpoint = f"getJurisdictionalOriginFromTSN?tsn={tsn}"
        data_format = data_format.lower()
        if data_format == "json":
            data = mr.make_request_for_content(self.base_url+endpoint)
            return xmltodict.parse(data)
        elif data_format == "xml":
            return mr.make_request_for_content(self.base_url+endpoint)
        else:
            return "Error: Format must be either json or xml."
            
    def get_kingdom_name_from_TSN(self, tsn, data_format="json"):
        """
        Returns the kingdom name for the TSN.
        
        Args:
        - tsn (int): The Taxonomic Serial Number (TSN).
        - data_format (str): The desired format for returned data. Defaults to json, but other option is xml.
        
        Returns:
        - dict or string (xml): A dictionary or XML string containing the data matching the TSN.
        """
        endpoint = f"getKingdomNameFromTSN?tsn={tsn}"
        data_format = data_format.lower()
        if data_format == "json":
            data = mr.make_request_for_content(self.base_url+endpoint)
            return xmltodict.parse(data)
        elif data_format == "xml":
            return mr.make_request_for_content(self.base_url+endpoint)
        else:
            return "Error: Format must be either json or xml."
            
    def get_other_sources_from_TSN(self, tsn, data_format="json"):
        """
        Returns a list of the other sources used for the TSN.
        
        Args:
        - tsn (int): The Taxonomic Serial Number (TSN).
        - data_format (str): The desired format for returned data. Defaults to json, but other option is xml.
        
        Returns:
        - dict or string (xml): A dictionary or XML string containing the data matching the TSN.
        """
        endpoint = f"getOtherSourcesFromTSN?tsn={tsn}"
        data_format = data_format.lower()
        if data_format == "json":
            data = mr.make_request_for_content(self.base_url+endpoint)
            return xmltodict.parse(data)
        elif data_format == "xml":
            return mr.make_request_for_content(self.base_url+endpoint)
        else:
            return "Error: Format must be either json or xml."
            
    def get_parent_TSN_from_TSN(self, tsn, data_format="json"):
        """
        Returns the parent TSN for the entered TSN.
        
        Args:
        - tsn (int): The Taxonomic Serial Number (TSN).
        - data_format (str): The desired format for returned data. Defaults to json, but other option is xml.
        
        Returns:
        - dict or string (xml): A dictionary or XML string containing the data matching the TSN.
        """
        endpoint = f"getParentTSNFromTSN?tsn={tsn}"
        data_format = data_format.lower()
        if data_format == "json":
            data = mr.make_request_for_content(self.base_url+endpoint)
            return xmltodict.parse(data)
        elif data_format == "xml":
            return mr.make_request_for_content(self.base_url+endpoint)
        else:
            return "Error: Format must be either json or xml."
            
    def get_publications_from_TSN(self, tsn, data_format="json"):
        """
        Returns a list of the pulications used for the TSN.
        
        Args:
        - tsn (int): The Taxonomic Serial Number (TSN).
        - data_format (str): The desired format for returned data. Defaults to json, but other option is xml.
        
        Returns:
        - dict or string (xml): A dictionary or XML string containing the data matching the TSN.
        """
        endpoint = f"getPublicationsFromTSN?tsn={tsn}"
        data_format = data_format.lower()
        if data_format == "json":
            data = mr.make_request_for_content(self.base_url+endpoint)
            return xmltodict.parse(data)
        elif data_format == "xml":
            return mr.make_request_for_content(self.base_url+endpoint)
        else:
            return "Error: Format must be either json or xml."
            
    def get_review_year_from_TSN(self, tsn, data_format="json"):
        """
        Returns the review year for the TSN.
        
        Args:
        - tsn (int): The Taxonomic Serial Number (TSN).
        - data_format (str): The desired format for returned data. Defaults to json, but other option is xml.
        
        Returns:
        - dict or string (xml): A dictionary or XML string containing the data matching the TSN.
        """
        endpoint = f"getReviewYearFromTSN?tsn={tsn}"
        data_format = data_format.lower()
        if data_format == "json":
            data = mr.make_request_for_content(self.base_url+endpoint)
            return xmltodict.parse(data)
        elif data_format == "xml":
            return mr.make_request_for_content(self.base_url+endpoint)
        else:
            return "Error: Format must be either json or xml."
            
    def get_scientific_name_from_TSN(self, tsn, data_format="json"):
        """
        Returns the scientific name for the TSN. Also returns the component parts (names and indicators) of the scientific name.
        
        Args:
        - tsn (int): The Taxonomic Serial Number (TSN).
        - data_format (str): The desired format for returned data. Defaults to json, but other option is xml.
        
        Returns:
        - dict or string (xml): A dictionary or XML string containing the data matching the TSN.
        """
        endpoint = f"getScientificNameFromTSN?tsn={tsn}"
        data_format = data_format.lower()
        if data_format == "json":
            data = mr.make_request_for_content(self.base_url+endpoint)
            return xmltodict.parse(data)
        elif data_format == "xml":
            return mr.make_request_for_content(self.base_url+endpoint)
        else:
            return "Error: Format must be either json or xml."
            
    def get_synonym_names_from_TSN(self, tsn, data_format="json"):
        """
        Returns a list of the synonyms (if any) for the TSN.
        
        Args:
        - tsn (int): The Taxonomic Serial Number (TSN).
        - data_format (str): The desired format for returned data. Defaults to json, but other option is xml.
        
        Returns:
        - dict or string (xml): A dictionary or XML string containing the data matching the TSN.
        """
        endpoint = f"getSynonymNamesFromTSN?tsn={tsn}"
        data_format = data_format.lower()
        if data_format == "json":
            data = mr.make_request_for_content(self.base_url+endpoint)
            return xmltodict.parse(data)
        elif data_format == "xml":
            return mr.make_request_for_content(self.base_url+endpoint)
        else:
            return "Error: Format must be either json or xml."
            
    def get_taxon_authorship_from_TSN(self, tsn, data_format="json"):
        """
        Returns the author information for the TSN.
        
        Args:
        - tsn (int): The Taxonomic Serial Number (TSN).
        - data_format (str): The desired format for returned data. Defaults to json, but other option is xml.
        
        Returns:
        - dict or string (xml): A dictionary or XML string containing the data matching the TSN.
        """
        endpoint = f"getTaxonAuthorshipFromTSN?tsn={tsn}"
        data_format = data_format.lower()
        if data_format == "json":
            data = mr.make_request_for_content(self.base_url+endpoint)
            return xmltodict.parse(data)
        elif data_format == "xml":
            return mr.make_request_for_content(self.base_url+endpoint)
        else:
            return "Error: Format must be either json or xml."
            
    def get_taxonomic_rank_name_from_TSN(self, tsn, data_format="json"):
        """
        Returns the kingdom and rank information for the TSN.
        
        Args:
        - tsn (int): The Taxonomic Serial Number (TSN).
        - data_format (str): The desired format for returned data. Defaults to json, but other option is xml.
        
        Returns:
        - dict or string (xml): A dictionary or XML string containing the data matching the TSN.
        """
        endpoint = f"getTaxonomicRankNameFromTSN?tsn={tsn}"
        data_format = data_format.lower()
        if data_format == "json":
            data = mr.make_request_for_content(self.base_url+endpoint)
            return xmltodict.parse(data)
        elif data_format == "xml":
            return mr.make_request_for_content(self.base_url+endpoint)
        else:
            return "Error: Format must be either json or xml."
            
    def get_taxonomic_usage_from_TSN(self, tsn, data_format="json"):
        """
        Returns the usage information for the TSN.
        
        Args:
        - tsn (int): The Taxonomic Serial Number (TSN).
        - data_format (str): The desired format for returned data. Defaults to json, but other option is xml.
        
        Returns:
        - dict or string (xml): A dictionary or XML string containing the data matching the TSN.
        """
        endpoint = f"getTaxonomicUsageFromTSN?tsn={tsn}"
        data_format = data_format.lower()
        if data_format == "json":
            data = mr.make_request_for_content(self.base_url+endpoint)
            return xmltodict.parse(data)
        elif data_format == "xml":
            return mr.make_request_for_content(self.base_url+endpoint)
        else:
            return "Error: Format must be either json or xml."
            
    def get_unacceptability_reason_from_TSN(self, tsn, data_format="json"):
        """
        Returns the unacceptability reason, if any, for the TSN.
        
        Args:
        - tsn (int): The Taxonomic Serial Number (TSN).
        - data_format (str): The desired format for returned data. Defaults to json, but other option is xml.
        
        Returns:
        - dict or string (xml): A dictionary or XML string containing the data matching the TSN.
        """
        endpoint = f"getUnacceptabilityReasonFromTSN?tsn={tsn}"
        data_format = data_format.lower()
        if data_format == "json":
            data = mr.make_request_for_content(self.base_url+endpoint)
            return xmltodict.parse(data)
        elif data_format == "xml":
            return mr.make_request_for_content(self.base_url+endpoint)
        else:
            return "Error: Format must be either json or xml."
            
            
    # HIERARCHY FUNCTIONS
    def get_full_hierarchy_from_TSN(self, tsn, data_format="json"):
        """
        Returns the Taxonomic Hierarchy from the kingdom to the requested scientific name, and the immediate children of the scientific name.
        
        Args:
        - tsn (int): The Taxonomic Serial Number (TSN).
        - data_format (str): The desired format for returned data. Defaults to json, but other option is xml.
        
        Returns:
        - dict or string (xml): A dictionary or XML string containing the data matching the TSN.
        """
        endpoint = f"getFullHierarchyFromTSN?tsn={tsn}"
        data_format = data_format.lower()
        if data_format == "json":
            data = mr.make_request_for_content(self.base_url+endpoint)
            return xmltodict.parse(data)
        elif data_format == "xml":
            return mr.make_request_for_content(self.base_url+endpoint)
        else:
            return "Error: Format must be either json or xml."
            
    def get_hierarchy_down_from_TSN(self, tsn, data_format="json"):
        """
        Returns a list of all the valid/accepted scientific names contained within a particular valid/accepted scientific name and their TSNs, limited to immediate children only. The result set will be found by comparing the search key to the parent TSN field.
        
        Args:
        - tsn (int): The Taxonomic Serial Number (TSN).
        - data_format (str): The desired format for returned data. Defaults to json, but other option is xml.
        
        Returns:
        - dict or string (xml): A dictionary or XML string containing the data matching the TSN.
        """
        endpoint = f"getHierarchyDownFromTSN?tsn={tsn}"
        data_format = data_format.lower()
        if data_format == "json":
            data = mr.make_request_for_content(self.base_url+endpoint)
            return xmltodict.parse(data)
        elif data_format == "xml":
            return mr.make_request_for_content(self.base_url+endpoint)
        else:
            return "Error: Format must be either json or xml."
            
    def get_hierarchy_up_from_TSN(self, tsn, data_format="json"):
        """
        Returns the parent of the TSN (i.e. the parent of the current scientific name in the taxonomic hierarchy) found by comparing the search key to the ITIS TSN field.
        
        Args:
        - tsn (int): The Taxonomic Serial Number (TSN).
        - data_format (str): The desired format for returned data. Defaults to json, but other option is xml.
        
        Returns:
        - dict or string (xml): A dictionary or XML string containing the data matching the TSN.
        """
        endpoint = f"getHierarchyUpFromTSN?tsn={tsn}"
        data_format = data_format.lower()
        if data_format == "json":
            data = mr.make_request_for_content(self.base_url+endpoint)
            return xmltodict.parse(data)
        elif data_format == "xml":
            return mr.make_request_for_content(self.base_url+endpoint)
        else:
            return "Error: Format must be either json or xml."
            
            
    # LIFE SCIENCE IDENTIFIER (LSID) FUNCTIONS
    def get_full_record_from_LSID(self, tsn, data_format="json"):
        """
        Returns the full ITIS record for the TSN in the LSID, found by comparing the TSN in the search key to the TSN field. Returns an empty result set if there is no match or the TSN is invalid.
        
        Args:
        - tsn (int): The Taxonomic Serial Number (TSN).
        - data_format (str): The desired format for returned data. Defaults to json, but other option is xml.
        
        Returns:
        - dict or string (xml): A dictionary or XML string containing the data matching the TSN.
        """
        endpoint = f"getFullRecordFromLSID?lsid=urn:lsid:itis.gov:itis_tsn:{tsn}"
        data_format = data_format.lower()
        if data_format == "json":
            data = mr.make_request_for_content(self.base_url+endpoint)
            return xmltodict.parse(data)
        elif data_format == "xml":
            return mr.make_request_for_content(self.base_url+endpoint)
        else:
            return "Error: Format must be either json or xml."
            
    def get_LSID_from_TSN(self, tsn, data_format="json"):
        """
        Gets the unique LSID for the TSN, or an empty result if there is no match.
        
        Args:
        - tsn (int): The Taxonomic Serial Number (TSN).
        - data_format (str): The desired format for returned data. Defaults to json, but other option is xml.
        
        Returns:
        - dict or string (xml): A dictionary or XML string containing the data matching the TSN.
        """
        endpoint = f"getLSIDFromTSN?tsn={tsn}"
        data_format = data_format.lower()
        if data_format == "json":
            data = mr.make_request_for_content(self.base_url+endpoint)
            return xmltodict.parse(data)
        elif data_format == "xml":
            return mr.make_request_for_content(self.base_url+endpoint)
        else:
            return "Error: Format must be either json or xml."
            
    def get_record_from_LSID(self, tsn, data_format="json"):
        """
        Gets the partial ITIS record for the TSN in the LSID, found by comparing the TSN in the search key to the TSN field. Returns an empty result set if there is no match or the TSN is invalid.
        
        Args:
        - tsn (int): The Taxonomic Serial Number (TSN).
        - data_format (str): The desired format for returned data. Defaults to json, but other option is xml.
        
        Returns:
        - dict or string (xml): A dictionary or XML string containing the data matching the TSN.
        """
        endpoint = f"getRecordFromLSID?lsid=urn:lsid:itis.gov:itis_tsn:{tsn}"
        data_format = data_format.lower()
        if data_format == "json":
            data = mr.make_request_for_content(self.base_url+endpoint)
            return xmltodict.parse(data)
        elif data_format == "xml":
            return mr.make_request_for_content(self.base_url+endpoint)
        else:
            return "Error: Format must be either json or xml."
            
    def get_TSN_from_LSID(self, tsn, data_format="json"):
        """
        Gets the TSN corresponding to the LSID, or an empty result if there is no match.
        
        Args:
        - tsn (int): The Taxonomic Serial Number (TSN).
        - data_format (str): The desired format for returned data. Defaults to json, but other option is xml.
        
        Returns:
        - dict or string (xml): A dictionary or XML string containing the data matching the TSN.
        """
        endpoint = f"getTSNFromLSID?lsid=urn:lsid:itis.gov:itis_tsn:{tsn}"
        data_format = data_format.lower()
        if data_format == "json":
            data = mr.make_request_for_content(self.base_url+endpoint)
            return xmltodict.parse(data)
        elif data_format == "xml":
            return mr.make_request_for_content(self.base_url+endpoint)
        else:
            return "Error: Format must be either json or xml."
            
            
    # META-INFORMATION FUNCTIONS
    def get_credibility_ratings(self, data_format="json"):
        """
        Provides a list of all the unique valid credibility rating values contained in the database.
        
        Args:
        - data_format (str): The desired format for returned data. Defaults to json, but other option is xml.
        
        Returns:
        - dict or string (xml): A dictionary or XML string containing the credibility rating values.
        """
        endpoint = "getCredibilityRatings"
        data_format = data_format.lower()
        if data_format == "json":
            data = mr.make_request_for_content(self.base_url+endpoint)
            return xmltodict.parse(data)
        elif data_format == "xml":
            return mr.make_request_for_content(self.base_url+endpoint)
        else:
            return "Error: Format must be either json or xml."
            
    def get_description(self, data_format="json"):
        """
        Provides a description of the web service and the ITIS database.
        
        Args:
        - data_format (str): The desired format for returned data. Defaults to json, but other option is xml.
        
        Returns:
        - dict or string (xml): A dictionary or XML string containing a description of the ITIS database.
        """
        endpoint = "getDescription"
        data_format = data_format.lower()
        if data_format == "json":
            data = mr.make_request_for_content(self.base_url+endpoint)
            return xmltodict.parse(data)
        elif data_format == "xml":
            return mr.make_request_for_content(self.base_url+endpoint)
        else:
            return "Error: Format must be either json or xml."
            
    def get_geographic_values(self, data_format="json"):
        """
        Provides a list of all the geographic values contained in the database.
        
        Args:
        - data_format (str): The desired format for returned data. Defaults to json, but other option is xml.
        
        Returns:
        - dict or string (xml): A dictionary or XML string containing the geographic values contained in the database.
        """
        endpoint = "getGeographicValues"
        data_format = data_format.lower()
        if data_format == "json":
            data = mr.make_request_for_content(self.base_url+endpoint)
            return xmltodict.parse(data)
        elif data_format == "xml":
            return mr.make_request_for_content(self.base_url+endpoint)
        else:
            return "Error: Format must be either json or xml."
            
    def get_jurisdiction_values(self, data_format="json"):
        """
        Provides a list of all the jurisdiction values contained in the database.
        
        Args:
        - data_format (str): The desired format for returned data. Defaults to json, but other option is xml.
        
        Returns:
        - dict or string (xml): A dictionary or XML string containing the jurisdiction values contained in the database.
        """
        endpoint = "getJurisdictionValues"
        data_format = data_format.lower()
        if data_format == "json":
            data = mr.make_request_for_content(self.base_url+endpoint)
            return xmltodict.parse(data)
        elif data_format == "xml":
            return mr.make_request_for_content(self.base_url+endpoint)
        else:
            return "Error: Format must be either json or xml."
            
    def get_jurisdictional_origin_values(self, data_format="json"):
        """
        Provides a list of all the origin values contained in the database.
        
        Args:
        - data_format (str): The desired format for returned data. Defaults to json, but other option is xml.
        
        Returns:
        - dict or string (xml): A dictionary or XML string containing the origin values contained in the database.
        """
        endpoint = "getJurisdictionalOriginValues"
        data_format = data_format.lower()
        if data_format == "json":
            data = mr.make_request_for_content(self.base_url+endpoint)
            return xmltodict.parse(data)
        elif data_format == "xml":
            return mr.make_request_for_content(self.base_url+endpoint)
        else:
            return "Error: Format must be either json or xml."
            
    def get_kingdom_names(self, data_format="json"):
        """
        Provides a list of all the unique kingdom names contained in the database, their kingdom IDs and their TSNs.
        
        Args:
        - data_format (str): The desired format for returned data. Defaults to json, but other option is xml.
        
        Returns:
        - dict or string (xml): A dictionary or XML string containing the kingdom names contained in the database.
        """
        endpoint = "getKingdomNames"
        data_format = data_format.lower()
        if data_format == "json":
            data = mr.make_request_for_content(self.base_url+endpoint)
            return xmltodict.parse(data)
        elif data_format == "xml":
            return mr.make_request_for_content(self.base_url+endpoint)
        else:
            return "Error: Format must be either json or xml."
            
    def get_last_change_date(self, data_format="json"):
        """
        Provides the date the ITIS database was last updated.
        
        Args:
        - data_format (str): The desired format for returned data. Defaults to json, but other option is xml.
        
        Returns:
        - dict or string (xml): A dictionary or XML string containing the date the database was last updated.
        """
        endpoint = "getLastChangeDate"
        data_format = data_format.lower()
        if data_format == "json":
            data = mr.make_request_for_content(self.base_url+endpoint)
            return xmltodict.parse(data)
        elif data_format == "xml":
            return mr.make_request_for_content(self.base_url+endpoint)
        else:
            return "Error: Format must be either json or xml."
            
    def get_rank_names(self, data_format="json"):
        """
        Provides a list of all the unique rank names contained in the database and their kingdom and rank ID values.
        
        Args:
        - data_format (str): The desired format for returned data. Defaults to json, but other option is xml.
        
        Returns:
        - dict or string (xml): A dictionary or XML string containing a list of all the unique rank names contained in the database.
        """
        endpoint = "getRankNames"
        data_format = data_format.lower()
        if data_format == "json":
            data = mr.make_request_for_content(self.base_url+endpoint)
            return xmltodict.parse(data)
        elif data_format == "xml":
            return mr.make_request_for_content(self.base_url+endpoint)
        else:
            return "Error: Format must be either json or xml."
            
    def get_vernacular_languages(self, data_format="json"):
        """
        Provides a list of the unique languages used in the vernacular table.
        
        Args:
        - data_format (str): The desired format for returned data. Defaults to json, but other option is xml.
        
        Returns:
        - dict or string (xml): A dictionary or XML string containing a list of all the unique languages used in the vernacular table.
        """
        endpoint = "getVernacularLanguages"
        data_format = data_format.lower()
        if data_format == "json":
            data = mr.make_request_for_content(self.base_url+endpoint)
            return xmltodict.parse(data)
        elif data_format == "xml":
            return mr.make_request_for_content(self.base_url+endpoint)
        else:
            return "Error: Format must be either json or xml."
          
          
          
          
          
          
          
          
    
          
    
            
            
            
            
            
            
    
           
            
            
       
    
         
         
         
         
         
         
         
         
         
         
         
         
         
         
         
         
          
          
           
            
    
       
       
          
          
          
          
          
            
    
            
    
            
    
         
         
           
         
           
          
          
        
          
          
    
    


from datetime import date
from typing import Optional
from .. helperFuncs import make_request as mr

class Coinmap:
    """
    A class for interacting with the Coinmap API.
    
    Attributes:
    - base_url: The base URL of the Coinmap API.
    - about: A short description of the API.
    """
    def __init__(self):
        self.base_url = "https://coinmap.org/api/v1/"
        self.about = "The CoinMap API is a free resource to access data about thousands of crypto merchants, ATMs, grocery stores, shops, cafes, and other venues. This API is really simple to use since it has a flat data structure, doesn't require authorization, and a well-described data format."
        
    def get_docs_url(self):
        """
        Returns the URL for the Coinmap API documentation.
        
        Args:
        - None
        
        Returns:
        - string: The URL for the API docs.
        """
        return "https://coinmap.org/api/"
        
    def get_venues(self, min_lat: Optional[float] = None, max_lat: Optional[float] = None, min_lon: Optional[float] = None,
                   max_lon: Optional[float] = None, query: Optional[str] = None, category: Optional[str] = None,
                   owner: Optional[str] = None, upvoter: Optional[str] = None, before: Optional[date] = None,
                   after: Optional[date] = None, promoted: Optional[bool] = None, limit: Optional[int] = None,
                   offset: Optional[int] = None):
        """
        Returns a list of venues with crypto ATMs.
        
        Args:
        - min_lat (float): Minimum latitude (>=)
        - max_lat (float): Maximum latitude (<=)
        - min_lon (float): Minimum longitude (>=)
        - max_lon (float): Maximum longitude (<=)
        - query (str): Substring search in venue names
        - category (str): Filter category
        - owner (str): Venue owner (userhash)
        - upvoter (str): User who upvoted the venue (userhash)
        - before (date): Only show venues created before YYYY-MM-DD
        - after (date): Only show venues created after YYYY-MM-DD
        - promoted (bool): Show promoted venues (unset = both)
        - limit (int): Limit number of results
        - offset (int): Skip first N results
        
        Returns:
        - dict: A dictionary containing a list of venues with crypto ATMs.
        """
        endpoint = "venues"
        params = {}

        if min_lat is not None:
            params['lat1'] = min_lat
        if max_lat is not None:
            params['lat2'] = max_lat
        if min_lon is not None:
            params['lon1'] = min_lon
        if max_lon is not None:
            params['lon2'] = max_lon
        if query is not None:
            params['query'] = query
        if category is not None:
            params['category'] = category
        if owner is not None:
            params['owner'] = owner
        if upvoter is not None:
            params['upvoter'] = upvoter
        if before is not None:
            params['before'] = before.strftime('%Y-%m-%d')
        if after is not None:
            params['after'] = after.strftime('%Y-%m-%d')
        if promoted is not None:
            params['promoted'] = promoted
        if limit is not None:
            params['limit'] = limit
        if offset is not None:
            params['offset'] = offset

        return mr.make_request_with_params(self.base_url + endpoint, params=params)
        
    def get_venue_by_id(self, venue_id):
        """
        Returns a venue matching the given id.
        
        Args:
        - venue_id (str): A unique number identifying a specific venue.
        
        Returns:
        - dict: A dictionary containing information about the venue matching the given id.
        """
        endpoint = f"venues/{venue_id}"
        return mr.make_request(self.base_url+endpoint)
        
    def get_comments_by_venue_id(self, venue_id):
        """
        Returns a list of comments about the venue matching the given id.
        
        Args:
        - venue_id (str): A unique number identifying a specific venue.
        
        Returns:
        - dict: A dictionary containing a list of comments about the venue matching the given id.
        """
        endpoint = f"venues/{venue_id}/comments"
        return mr.make_request(self.base_url+endpoint)
        
    def get_ratings_by_venue_id(self, venue_id):
        """
        Returns ratings about the venue matching the given id.
        
        Args:
        - venue_id (str): A unique number identifying a specific venue.
        
        Returns:
        - dict: A dictionary containing ratings about the venue matching the given id.
        """
        endpoint = f"venues/{venue_id}/ratings"
        return mr.make_request(self.base_url+endpoint)
        
    def get_atm_operators(self):
        """
        Returns a list of ATM operators.
        
        Args:
        - None
        
        Returns:
        - dict: A dictionary containing a list of ATM operators.
        """
        endpoint = "atm-operators"
        return mr.make_request(self.base_url+endpoint)
        
    def get_coins(self):
        """
        Returns a list of coins.
        
        Args:
        - None
        
        Returns:
        - dict: A dictionary containing a list of coins.
        """
        endpoint = "coins"
        return mr.make_request(self.base_url+endpoint)

# LISTED IN THE DOCS AS A VALID ENDPOINT, BUT IT THROWS UP AN ERROR. POSSIBLY DEPRECATED.    
#    def get_atm_providers(self):
#        """
#        Returns a list of ATM providers.
#        
#        Args:
#        - None
#        
#        Returns:
#        - dict: A dictionary containing a list of ATM providers.
#        """
#        endpoint = "providers"
#        return mr.make_request(self.base_url+endpoint)
        
    
    
        
        
        


from .. helperFuncs import make_request as mr

class Dictum:
    """
    A class for interacting with the Dictum API.
    
    Attributes:
    - base_url: The base URL of the API.
    - about: A short description of the API.
    """
    def __init__(self):
        self.base_url = "https://api.fisenko.net/v1/"
        self.about = "Dictum API provides a programmatic way to access the most inspiring expressions of humanity."
        
    def get_docs_url(self):
        """
        Returns the URL for the Dictum API documentation.
        
        Args:
        - None
        
        Returns:
        -string: The URL for the API docs.
        """
        return "https://api.fisenko.net/swagger-ui/"
        
    def get_inspiring_authors(self, language="en", search_query=None, offset=0, limit=0):
        """
        Returns list of authors featured in the API based on the given parameters.
        
        Args:
        - language (str): The language of the author. Default is en (English). Only other value supported is ru (Russian).
        - search_query (str): The search query matching the author's name. Default is None.
        - offset (int): The page offset for the list of authors. Default is 0.
        - limit (int): The number of items.
        
        Returns:
        - dict: A dictionary containing the authors matching the given parameters, along with the unique identifier for the author.
        """
        if search_query is not None:
            endpoint = f"authors/{language}?query={search_query}&offset={offset}&limit={limit}"
        else:
            endpoint = f"authors/{language}?&offset={offset}&limit={limit}"
        return mr.make_request(self.base_url+endpoint)
        
    def get_inspiring_author_by_id(self, authorID, language="en"):
        """
        Returns the author matching the unique identifier.
        
        Args:
        - language (str): The language of the author. Defaults to en (English). Only other value supported is ru (Russian).
        - authorID (str): A unique hexadecimal string identifying a specific author in the API.
        
        Returns:
        - dict: A dictionary containing the author matching the given id.
        """
        endpoint = f"authors/{language}/{authorID}"
        return mr.make_request(self.base_url+endpoint)
        
    def get_inspiring_quotes_by_author_id(self, authorID, language="en"):
        """
        Returns quotes from an author matching the unique identifier.
        
        Args:
        - language (str): The language of the author. Defaults to en (English). Only other value supported is ru (Russian).
        - authorID (str): A unique hexadecimal string identifying a specific author in the API.
        
        Returns:
        - dict: A dictionary containing quotes from the author matching the given id.
        """
        endpoint = f"authors/{language}/{authorID}/quotes"
        return mr.make_request(self.base_url+endpoint)
        
    def get_languages(self):
      """
      Returns a list of author languages used in the API.
      
      Args:
      - None
      
      Returns:
      - dict: A dictionary containing a list of languages (as of this time, only Russian and English)
      """
      endpoint = "languages"
      return mr.make_request(self.base_url+endpoint)
      
    def get_inspiring_quotes(self, language="en", search_query=None, offset=0, limit=0):
        """
        Returns list of quotes matching the given parameters.
        
        Args:
        - language (str): The language of the author. Default is en (English). Only other value supported is ru (Russian).
        - search_query (str): The search query matching the quote. Default is None.
        - offset (int): The page offset for the list of quotes. Default is 0.
        - limit (int): The number of items.
        
        Returns:
        - dict: A dictionary containing the quotes matching the given parameters, along with the unique identifier for the author.
        """
        if search_query is not None:
            endpoint = f"quotes/{language}?query={search_query}&offset={offset}&limit={limit}"
        else:
            endpoint = f"quotes/{language}?&offset={offset}&limit={limit}"
        return mr.make_request(self.base_url+endpoint)
        
    def get_random_quote(self, language="en"):
        """
        Returns a random quote.
        
        Args:
        - language (str): Language of the quote. Defaults to en (English). Only other value supported is ru (Russian).
        
        Returns:
        - dict: A dictionary containing the random quote.
        """
        endpoint = f"quotes/{language}/random"
        return mr.make_request(self.base_url+endpoint)
        
    def get_inspiring_quote_by_quote_id(self, quoteID, language="en"):
        """
        Returns a quote  matching the unique identifier.
        
        Args:
        - language (str): The language of the author. Defaults to en (English). Only other value supported is ru (Russian).
        - quoteID (str): A unique hexadecimal string identifying a specific quote in the API.
        
        Returns:
        - dict: A dictionary containing a quote matching the given id.
        """
        endpoint = f"quotes/{language}/{quoteID}"
        return mr.make_request(self.base_url+endpoint)
        
    def get_number_of_quotes_and_authors(self, language="en"):
        """
        Returns the number of quotes and authors in the API for the given language.
        
        Args:
        - language (str). The language of the quotes. Defaults to en (English). Only other value supported is ru (Russian).
        
        Returns:
        - dict: Dictionary containing the number of authors and quotes for the given language.
        """
        endpoint = f"statistics/{language}"
        return mr.make_request(self.base_url+endpoint)
        
       
    
      
    
      
        


import requests

from .. helperFuncs import make_request as mr

class NagerDate:
    """
    A class to interact with the Nager.Date API.
    
    Attributes:
    - base_url: The base URL of the API.
    - about: A short description of the API.
    """
    def __init__(self):
        self.base_url = "https://date.nager.at/api/v3/"
        self.about = "The Nager.Date API provides a simple way to query the holidays of over 100 countries. It is also possible to query long weekends."
        
    def get_docs_url(self):
        """
        Returns the URL for the Nager.Date API documentation.
        
        Args:
        - None
        
        Returns:
        - string: The URL for the API docs.
        """
        return "https://date.nager.at/swagger/index.html"
        
    def get_country_info(self, countryCode):
        """
        Returns information about a given country.
        
        Args:
        - countryCode (str): The code representing one of over a hundred countries supported by the API.
        
        Returns:
        - dict: A dictionary containing information about the given country.
        """
        endpoint = f"CountryInfo/{countryCode}"
        return mr.make_request(self.base_url+endpoint)
        
    def get_available_countries(self):
        """
        Returns all the countries available in the API.
        
        Args:
        - None
        
        Returns:
        - list: A list containing all the countries available in the API.
        """
        endpoint = "AvailableCountries"
        return mr.make_request(self.base_url+endpoint)
        
    def get_long_weekends_by_country(self, year, countryCode):
        """
        Returns a list of long weekends for a given country in a given year.
        
        Args:
        - year (int): The year in question.
        - countryCode (str): The code representing one of over a hundred countries supported by the API.
        
        Returns:
        - list: A list containing the long weekends for a given country in a given year.
        """
        endpoint = f"LongWeekend/{year}/{countryCode}"
        return mr.make_request(self.base_url+endpoint)
        
    def get_public_holidays(self, year, countryCode):
        """
        Returns a list of public holidays for a given country in a given year.
        
        Args:
        - year (int): The year in question.
        - countryCode (str): The code representing one of over a hundred countries supported by the API.
        
        Returns:
        - list: A list containing the public holidays for a given country in a given year.
        """
        endpoint = f"PublicHolidays/{year}/{countryCode}"
        return mr.make_request(self.base_url+endpoint)
        
    def is_today_a_public_holiday(self, countryCode, utc_offset=0):
        """
        Returns True if today (UTC time zone) is a public holiday for a given country or False if it is not.
        
        Args:
        - countryCode (str): The code representing one of over a hundred countries supported by the API.
        - utc_offset (int): Integer offset from UTC time zone to adjust for one's own time zone.
        
        Returns:
        - bool: True if today (UTC time zone) is a public holiday for the given country or False if it is not.
        """
        endpoint = f"IsTodayPublicHoliday/{countryCode}?offset={utc_offset}"
        url = self.base_url+endpoint
        response = requests.get(url)
        if response.status_code == 200:
            return True
        elif response.status_code == 204:
            return False
        elif response.status_code == 400:
            return "Validation error"
        elif response.status_code == 404:
            return "Country code is unknown"
            
    def get_public_holdiays_for_next_365(self, countryCode):
        """
        Returns a list of the upcoming public holidays for a given country for the next 365 days.
        
        Args:
        - countryCode (str): The code representing one of over a hundred countries supported by the API.
        
        Returns:
        - list: A list containing the upcoming public holidays for a given country for the next 365 days.
        """
        endpoint = f"NextPublicHolidays/{countryCode}"
        return mr.make_request(self.base_url+endpoint)
        
    def get_public_holidays_worldwide_for_next_7(self):
        """
        Returns a list of all the public holidays worldwide for the next 7 days.
        
        Args:
        - None
        
        Returns:
        - list: A list containing all the upcoming public holidays worldwide for the next 7 days.
        """
        endpoint = "NextPublicHolidaysWorldwide"
        return mr.make_request(self.base_url+endpoint)
     
  
            
    
           
        
    
        
        
        
        


from . nokey_apis import nokey_apis

def get_api_list():
    """
    Prints out the APIs in a visually pleasing format with headings for the categories 
    and the individual APIs under the respective headings.

    Args:
    - None
    
    Returns:
    - string: A list of the apis supported by the nokey package.
    """

    # ANSI escape codes for colors
    green_color = "\033[92m"  # Green
    blue_color = "\033[94m"   # Blue
    reset_color = "\033[0m"   # Reset color to default

    for category, apis in nokey_apis.items():
        print(f"{green_color}{category}{reset_color}")
        print("-" * (len(category) + 6))  # Line for aesthetics
        for api in apis:
            print(f"{blue_color}{api}{reset_color}")
        print()  # Empty line for better readability between categories
    return ""




from .. helperFuncs import make_request as mr


class NationalWeatherService:
    """
    A class to interact with the National Weather Service API.
    
    Attributes:
    - base_url: The base URL of the National Weather Service API.
    - about: A short description of the API.
    """
    def __init__(self):
        self.base_url = "https://api.weather.gov/"
        self.about = "The National Weather Service (NWS) API allows developers access to critical forecasts, alerts, and observations, along with other weather data."
        
    def get_docs_url(self):
        """
        Returns URL for API docs.
        
        Args:
        - None
        
        Returns:
        - string: URL for API documentation.
        """
        return "https://www.weather.gov/documentation/services-web-api"       
    
    def get_alerts(self):
        """
        Returns a collection of weather alerts.

        Args:
        - None

        Returns:
        - dict: Inforation on weather alerts
        """
        endpoint = "alerts"
        return mr.make_request(self.base_url+endpoint)
        
    def get_active_alerts(self):
        """
        Returns a collection of currently active weather alerts.

        Args:
        - None

        Returns:
        - dict: Inforation on currently active weather alerts
        """
        endpoint = "alerts/active"
        return mr.make_request(self.base_url+endpoint)
        
    def get_active_alerts_count(self):
        """
        Returns the count of currently active weather alerts broken down by various categories.

        Args:
        - None

        Returns:
        - dict: Count of currently active weather alerts broke down by categories.
        """
        endpoint = "alerts/active/count"
        return mr.make_request(self.base_url+endpoint)
        
    def get_active_alerts_by_zone(self, zoneID):
        """
        Returns active alerts for the given NWS public zone or county.
        
        Args:
        - zoneID (str): NWS public zone/county identifier
        
        Returns:
        - dict: Information of active alerts in the zone specified by zoneID.
        """
        endpoint = f"alerts/active/zone/{zoneID}"
        return mr.make_request(self.base_url+endpoint)
    
    def get_active_alerts_by_area(self, area):
        """
        Returns active alerts for the given area (state or marine area).
        
        Args:
        - area (str): State or marine area ID
        
        Returns:
        - dict: Information of active alerts in the area specified.
        """
        endpoint = f"alerts/active/area/{area}"
        return mr.make_request(self.base_url+endpoint)
    
    def get_active_alerts_by_region(self, region):
        """
        Returns active alerts for the given marine region ID. Available values : AL, AT, GL, GM, PA, PI."
        
        Args:
        - region (str): Marine region ID. Available values : AL, AT, GL, GM, PA, PI.
        
        Returns:
        - dict: Information of active alerts in the area specified.
        """
        endpoint = f"alerts/active/region/{region}"
        return mr.make_request(self.base_url+endpoint)
        
    def get_alert_types(self):
        """
        Returns a list of alerts types.

        Args:
        - None

        Returns:
        - dict: A dictionary containing a list of alert types.
        """
        endpoint = "alerts/types"
        return mr.make_request(self.base_url+endpoint)
        
    def get_alert_by_id(self, ID):
        """
        Returns a specific alert record.

        Args:
        - ID (str): Alert identifier.

        Returns:
        - dict: A dictionary containing information about a specific alert.
        """
        endpoint = f"alerts/{ID}"
        return mr.make_request(self.base_url+endpoint)
        
    def get_cwsu_metadata(self, cwsuID):
        """
        Returns metadata about a Center Weather Service Unit.

        Args:
        - cwsuID (str): NWS CWSU ID. Available values : ZAB, ZAN, ZAU, ZBW, ZDC, ZDV, ZFA, ZFW, ZHU, ZID, ZJX, ZKC, ZLA, ZLC, ZMA, ZME, ZMP, ZNY, ZOA, ZOB, ZSE, ZTL.

        Returns:
        - dict: Metadata about a cwsu.
        """
        endpoint = f"aviation/cwsus/{cwsuID}"
        return mr.make_request(self.base_url+endpoint)
        
    def get_cwsu_advisories(self, cwsuID):
        """
        Returns a list of Center Weather Advisories from a CWSU.

        Args:
        - cwsuID (str): NWS CWSU ID. Available values : ZAB, ZAN, ZAU, ZBW, ZDC, ZDV, ZFA, ZFW, ZHU, ZID, ZJX, ZKC, ZLA, ZLC, ZMA, ZME, ZMP, ZNY, ZOA, ZOB, ZSE, ZTL.

        Returns:
        - dict: Dictionary containing list of  Center Weather Advisories from a CWSU.
        """
        endpoint = f"aviation/cwsus/{cwsuID}/cwas"
        return mr.make_request(self.base_url+endpoint)
    
    def get_cwsu_advisories_by_date_and_seq(self, cwsuID, date, sequence):
        """
        Returns a list of Center Weather Advisories from a CWSU.

        Args:
        - cwsuID (str): NWS CWSU ID. Available values : ZAB, ZAN, ZAU, ZBW, ZDC, ZDV, ZFA, ZFW, ZHU, ZID, ZJX, ZKC, ZLA, ZLC, ZMA, ZME, ZMP, ZNY, ZOA, ZOB, ZSE, ZTL.
        - date (str): (YYYY-MM-DD format) Date of advisory.
        - sequence (int): Sequence number
        
        Returns:
        - dict: Dictionary containing list of Center Weather Advisories from a CWSU.
        """
        endpoint = f"aviation/cwsus/{cwsuID}/cwas/{date}/{sequence}"
        return mr.make_request(self.base_url+endpoint)

# DON'T YET KNOW HOW THIS WORKS. ARGS ARE QUERIES, NOT PATHS      
#    def get_sigmets(self, start_time, end_time, date, atsu, sequence):
#        """
#        Returns a list of SIGMET/AIRMETs.
#
#        Args:
#        - start_time (str): Start time.
#        - end_time (str): End time.
#        - date (str): (YYYY-MM-DD format) Date
#        - atsu (str): ATSU identifier
#        - sequence (str): SIGMET sequence number
#
#        Returns:
#        - dict: Metadata about a cwsu.
#        """
#        endpoint = f"aviation/sigmet?{start_time}?{end_time}?{date}?{atsu}?{sequence}"
#        return mr.make_request(self.base_url+endpoint)

    def get_sigmets_by_atsu(self, atsu):
        """
        Returns a list of SIGMET/AIRMETs for the specified ATSU.

        Args:
        - atsu (str): ATSU identifier.
        
        Returns:
        - dict: Dictionary containing list of SIGMET/AIRMETs for the specified ATSU.
        """
        endpoint = f"aviation/sigmets/{atsu}"
        return mr.make_request(self.base_url+endpoint)

    def get_sigmets_by_atsu_and_date(self, atsu, date):
        """
        Returns a list of SIGMET/AIRMETs for the specified ATSU for the specified date.

        Args:
        - atsu (str): ATSU identifier.
        - date (str): Date (YYYY-MM-DD format)
        
        Returns:
        - dict: Dictionary containing list of SIGMET/AIRMETs for the specified ATSU and specified date.
        """
        endpoint = f"aviation/sigmets/{atsu}/{date}"
        return mr.make_request(self.base_url+endpoint)
        
    def get_specific_sigmet(self, atsu, date, time):
        """
        Returns a list of SIGMET/AIRMETs for the specified ATSU for the specified date.

        Args:
        - atsu (str): ATSU identifier.
        - date (str): Date (YYYY-MM-DD format)
        - time (str): Time (HHMM format). This time is always specified in UTC (Zulu) time.
        
        Returns:
        - dict: Dictionary containing a specific SIGMET/AIRMET.
        """
        endpoint = f"aviation/sigmets/{atsu}/{date}/{time}"
        return mr.make_request(self.base_url+endpoint)
        
    def get_glossary(self):
        """
        Returns glossary terms.

        Args:
        - None

        Returns:
        - dict: A dictionary containing a list of glossary terms and definitions.
        """
        endpoint = "glossary"
        return mr.make_request(self.base_url+endpoint)
        
    def get_raw_data_by_gridpoints(self, wfo, x, y):
        """
        Returns raw numerical forecast data for a 2.5km grid area.

        Args:
        - wfo (str): Forecast office ID. Available values : AKQ, ALY, BGM, BOX, BTV, BUF, CAE, CAR, CHS, CLE, CTP, GSP, GYX, ILM, ILN, LWX, MHX, OKX, PBZ, PHI, RAH, RLX, RNK, ABQ, AMA, BMX, BRO, CRP, EPZ, EWX, FFC, FWD, HGX, HUN, JAN, JAX, KEY, LCH, LIX, LUB, LZK, MAF, MEG, MFL, MLB, MOB, MRX, OHX, OUN, SHV, SJT, SJU, TAE, TBW, TSA, ABR, APX, ARX, BIS, BOU, CYS, DDC, DLH, DMX, DTX, DVN, EAX, FGF, FSD, GID, GJT, GLD, GRB, GRR, ICT, ILX, IND, IWX, JKL, LBF, LMK, LOT, LSX, MKX, MPX, MQT, OAX, PAH, PUB, RIW, SGF, TOP, UNR, BOI, BYZ, EKA, FGZ, GGW, HNX, LKN, LOX, MFR, MSO, MTR, OTX, PDT, PIH, PQR, PSR, REV, SEW, SGX, SLC, STO, TFX, TWC, VEF, AER, AFC, AFG, AJK, ALU, GUM, HPA, HFO, PPG, STU, NH1, NH2, ONA, ONP
        - x (int): Forecast grid X coordinate.
        - y (int): Forecast grid Y coordinate.
        
        Returns:
        - dict: Dictionary containing the raw numerical forcast data.
        """
        endpoint = f"gridpoints/{wfo}/{x},{y}"
        return mr.make_request(self.base_url+endpoint)
        
    def get_textual_forecast_by_gridpoints(self, wfo, x, y):
        """
        Returns a textual forecast for a 2.5km grid area.

        Args:
        - wfo (str): Forecast office ID. Available values : AKQ, ALY, BGM, BOX, BTV, BUF, CAE, CAR, CHS, CLE, CTP, GSP, GYX, ILM, ILN, LWX, MHX, OKX, PBZ, PHI, RAH, RLX, RNK, ABQ, AMA, BMX, BRO, CRP, EPZ, EWX, FFC, FWD, HGX, HUN, JAN, JAX, KEY, LCH, LIX, LUB, LZK, MAF, MEG, MFL, MLB, MOB, MRX, OHX, OUN, SHV, SJT, SJU, TAE, TBW, TSA, ABR, APX, ARX, BIS, BOU, CYS, DDC, DLH, DMX, DTX, DVN, EAX, FGF, FSD, GID, GJT, GLD, GRB, GRR, ICT, ILX, IND, IWX, JKL, LBF, LMK, LOT, LSX, MKX, MPX, MQT, OAX, PAH, PUB, RIW, SGF, TOP, UNR, BOI, BYZ, EKA, FGZ, GGW, HNX, LKN, LOX, MFR, MSO, MTR, OTX, PDT, PIH, PQR, PSR, REV, SEW, SGX, SLC, STO, TFX, TWC, VEF, AER, AFC, AFG, AJK, ALU, GUM, HPA, HFO, PPG, STU, NH1, NH2, ONA, ONP
        - x (int): Forecast grid X coordinate.
        - y (int): Forecast grid Y coordinate.
        
        Returns:
        - dict: Dictionary containing the textual forcast data.
        """
        endpoint = f"gridpoints/{wfo}/{x},{y}/forecast"
        return mr.make_request(self.base_url+endpoint)
        
    def get_textual_hourly_forecast_by_gridpoints(self, wfo, x, y):
        """
        Returns a textual hourly forecast for a 2.5km grid area.

        Args:
        - wfo (str): Forecast office ID. Available values : AKQ, ALY, BGM, BOX, BTV, BUF, CAE, CAR, CHS, CLE, CTP, GSP, GYX, ILM, ILN, LWX, MHX, OKX, PBZ, PHI, RAH, RLX, RNK, ABQ, AMA, BMX, BRO, CRP, EPZ, EWX, FFC, FWD, HGX, HUN, JAN, JAX, KEY, LCH, LIX, LUB, LZK, MAF, MEG, MFL, MLB, MOB, MRX, OHX, OUN, SHV, SJT, SJU, TAE, TBW, TSA, ABR, APX, ARX, BIS, BOU, CYS, DDC, DLH, DMX, DTX, DVN, EAX, FGF, FSD, GID, GJT, GLD, GRB, GRR, ICT, ILX, IND, IWX, JKL, LBF, LMK, LOT, LSX, MKX, MPX, MQT, OAX, PAH, PUB, RIW, SGF, TOP, UNR, BOI, BYZ, EKA, FGZ, GGW, HNX, LKN, LOX, MFR, MSO, MTR, OTX, PDT, PIH, PQR, PSR, REV, SEW, SGX, SLC, STO, TFX, TWC, VEF, AER, AFC, AFG, AJK, ALU, GUM, HPA, HFO, PPG, STU, NH1, NH2, ONA, ONP
        - x (int): Forecast grid X coordinate.
        - y (int): Forecast grid Y coordinate.
        
        Returns:
        - dict: Dictionary containing the textual hourly forcast data.
        """
        endpoint = f"gridpoints/{wfo}/{x},{y}/forecast/hourly"
        return mr.make_request(self.base_url+endpoint)
        
    def get_observation_stations_by_gridpoints(self, wfo, x, y):
        """
        Returns a list of observation stations usable for a given 2.5km grid area.

        Args:
        - wfo (str): Forecast office ID. Available values : AKQ, ALY, BGM, BOX, BTV, BUF, CAE, CAR, CHS, CLE, CTP, GSP, GYX, ILM, ILN, LWX, MHX, OKX, PBZ, PHI, RAH, RLX, RNK, ABQ, AMA, BMX, BRO, CRP, EPZ, EWX, FFC, FWD, HGX, HUN, JAN, JAX, KEY, LCH, LIX, LUB, LZK, MAF, MEG, MFL, MLB, MOB, MRX, OHX, OUN, SHV, SJT, SJU, TAE, TBW, TSA, ABR, APX, ARX, BIS, BOU, CYS, DDC, DLH, DMX, DTX, DVN, EAX, FGF, FSD, GID, GJT, GLD, GRB, GRR, ICT, ILX, IND, IWX, JKL, LBF, LMK, LOT, LSX, MKX, MPX, MQT, OAX, PAH, PUB, RIW, SGF, TOP, UNR, BOI, BYZ, EKA, FGZ, GGW, HNX, LKN, LOX, MFR, MSO, MTR, OTX, PDT, PIH, PQR, PSR, REV, SEW, SGX, SLC, STO, TFX, TWC, VEF, AER, AFC, AFG, AJK, ALU, GUM, HPA, HFO, PPG, STU, NH1, NH2, ONA, ONP
        - x (int): Forecast grid X coordinate.
        - y (int): Forecast grid Y coordinate.
        
        Returns:
        - dict: Dictionary containing a list of observation stations usable for a given 2.5km grid area.
        """
        endpoint = f"gridpoints/{wfo}/{x},{y}/stations"
        return mr.make_request(self.base_url+endpoint)
        
    def get_observations_by_station(self, stationID):
        """
        Returns a list of observations for a given station.

        Args:
        - stationID (str): Observation station ID.
        
        Returns:
        - dict: Dictionary containing list of observations for a given station.
        """
        endpoint = f"stations/{stationID}/observations"
        return mr.make_request(self.base_url+endpoint)
        
    def get_station_observation_by_time(self, stationID, time):
        """
        Returns a single observation.

        Args:
        - stationID (str): Observation station ID.
        - time (str): Timestamp of requested observation.
        
        Returns:
        - dict: Dictionary containing single observation for a given station at a given time.
        """
        endpoint = f"stations/{stationID}/observations/{time}"
        return mr.make_request(self.base_url+endpoint)
        
    def get_tafs_by_station(self, stationID):
        """
        Returns Terminal Aerodrome Forecasts for the specified airport station.

        Args:
        - stationID (str): Observation station ID.
        
        Returns:
        - dict: Dictionary containing TAFS for given station.
        """
        endpoint = f"stations/{stationID}/tafs"
        return mr.make_request(self.base_url+endpoint)
        
    def get_taf_by_date_time(self, stationID, date, time):
        """
        Returns a single Terminal Aerodrome Forecast..

        Args:
        - stationID (str): Observation station ID.
        - date (str): Date (YYYY-MM-DD format)
        - time (str) Time (HHMM format). This time is always specified in UTC (Zulu) time.
        
        Returns:
        - dict: Dictionary containing TAF.
        """
        endpoint = f"stations/{stationID}/tafs/{date}/{time}"
        return mr.make_request(self.base_url+endpoint)
        
    def get_stations(self):
        """
        Returns a list of observation stations.

        Args:
        - None

        Returns:
        - dict: A dictionary containing a list of observation stations.
        """
        endpoint = "stations"
        return mr.make_request(self.base_url+endpoint)
        
    def get_station_metadata(self, stationID):
        """
        Returns metadata about a given observation station.

        Args:
        - stationID (str): Observation station ID.
        
        Returns:
        - dict: Dictionary containing metadata for given station.
        """
        endpoint = f"stations/{stationID}"
        return mr.make_request(self.base_url+endpoint)
        
    def get_NWS_office_metadata(self, officeID):
        """
        Returns metadata about a NWS forecast office.

        Args:
        - officeID (str): NWS office ID.
        
        Returns:
        - dict: Dictionary containing metadata for given NWS office.
        """
        endpoint = f"offices/{officeID}"
        return mr.make_request(self.base_url+endpoint)
        
    def get_NWS_office_headline_by_headline_id(self, officeID, headlineID):
        """
        Returns a specific news headline for a given NWS office.

        Args:
        - officeID (str): NWS office ID.
        - headlineID (str): Headline record ID.
        
        Returns:
        - dict: Dictionary containing headline for given NWS office and given headline ID.
        """
        endpoint = f"offices/{officeID}/headlines/{headlineID}"
        return mr.make_request(self.base_url+endpoint)
        
    def get_NWS_office_headlines(self, officeID):
        """
        Returns a list of news headlines for a given NWS office.

        Args:
        - officeID (str): NWS office ID.
        
        Returns:
        - dict: Dictionary containing headlines for given NWS office.
        """
        endpoint = f"offices/{officeID}/headlines"
        return mr.make_request(self.base_url+endpoint)

# HAVEN'T FIGURED THIS ONE OUT, HOW TO PASS POINT ARG AS LAT AND LONG      
#    def get_metadata_by_point(self, point):
#        """
#        Returns metadata about a given latitude/longitude point.
#
#        Args:
#        - point (str): Point(latitude,longitude)
#        
#        Returns:
#        - dict: Dictionary containing metadata for given point.
#        """
#        endpoint = f"points/{point}"
#        return mr.make_request(self.base_url+endpoint)

    def get_radar_servers(self):
        """
        Returns a list of radar servers.

        Args:
        - None

        Returns:
        - dict: A dictionary containing a list of radar servers.
        """
        endpoint = "radar/servers"
        return mr.make_request(self.base_url+endpoint)
        
    def get_radar_server_metadata_by_id(self, serverID):
        """
        Returns metadata about a given radar server.

        Args:
        - serverID (str): Radar server ID.
        
        Returns:
        - dict: Dictionary containing metadata for given radar server.
        """
        endpoint = f"radar/servers/{serverID}"
        return mr.make_request(self.base_url+endpoint)
        
    def get_radar_stations(self):
        """
        Returns a list of radar stations.

        Args:
        - None

        Returns:
        - dict: A dictionary containing a list of radar stations.
        """
        endpoint = "radar/stations"
        return mr.make_request(self.base_url+endpoint)
        
    def get_radar_station_metadata_by_id(self, stationID):
        """
        Returns metadata about a given radar station.

        Args:
        - stationID (str): Radar station ID.
        
        Returns:
        - dict: Dictionary containing metadata for given radar station.
        """
        endpoint = f"radar/stations/{stationID}"
        return mr.make_request(self.base_url+endpoint)
        
    def get_radar_station_alarms_metadata_by_id(self, stationID):
        """
        Returns metadata about a given radar station alarms.

        Args:
        - stationID (str): Radar station ID.
        
        Returns:
        - dict: Dictionary containing metadata for given radar station alarms.
        """
        endpoint = f"radar/stations/{stationID}/alarms"
        return mr.make_request(self.base_url+endpoint)
        
    def get_radar_queue_metadata(self, host):
        """
        Returns metadata about a given radar queue.

        Args:
        - host (str): LDM host.
        
        Returns:
        - dict: Dictionary containing metadata for given radar queue.
        """
        endpoint = f"radar/queues/{host}"
        return mr.make_request(self.base_url+endpoint)
        
    def get_radar_wind_profiler_metadata(self, stationID):
        """
        Returns metadata about a given radar wind profiler.

        Args:
        - stationID (str): Profiler station ID.
        
        Returns:
        - dict: Dictionary containing metadata for given radar wind profiler.
        """
        endpoint = f"radar/profilers/{stationID}"
        return mr.make_request(self.base_url+endpoint)
        
    def get_text_products(self):
        """
        Returns a list of text products.

        Args:
        - None

        Returns:
        - dict: A dictionary containing a list of text products.
        """
        endpoint = "products"
        return mr.make_request(self.base_url+endpoint)
        
    def get_text_product_locations(self):
        """
        Returns a list of valid text product issuance locations.

        Args:
        - None

        Returns:
        - dict: A dictionary containing a list of valid text product issuance locations.
        """
        endpoint = "products/locations"
        return mr.make_request(self.base_url+endpoint)
        
    def get_text_product_types(self):
        """
        Returns a list of valid text product types and codes.

        Args:
        - None

        Returns:
        - dict: A dictionary containing a list of valid text product types and codes.
        """
        endpoint = "products/types"
        return mr.make_request(self.base_url+endpoint)
        
    def get_text_product_by_id(self, productID):
        """
        Returns a specific text product.

        Args:
        - productID (str): Text product ID.
        
        Returns:
        - dict: Dictionary containing specific text product.
        """
        endpoint = f"products/{productID}"
        return mr.make_request(self.base_url+endpoint)
        
    def get_text_products_by_type(self, typeID):
        """
        Returns a list of text products of a given type.

        Args:
        - typeID (str): Text product type ID.
        
        Returns:
        - dict: Dictionary containing list of text products of given type.
        """
        endpoint = f"products/types/{typeID}"
        return mr.make_request(self.base_url+endpoint)
        
    def get_text_product_locations_by_type(self, typeID):
        """
        Returns a list of valid text product issuance locations for a given product type.

        Args:
        - typeID (str): Text product type ID.
        
        Returns:
        - dict: Dictionary containing list of valid text product issuance locations for a given type.
        """
        endpoint = f"products/types/{typeID}/locations"
        return mr.make_request(self.base_url+endpoint)
        
    def get_text_product_types_by_location(self, locationID):
        """
        Returns a list of valid text product types for a given issuance location.

        Args:
        - locationID (str): Location ID.
        
        Returns:
        - dict: Dictionary containing list of valid text product types for a given location.
        """
        endpoint = f"products/locations/{locationID}/types"
        return mr.make_request(self.base_url+endpoint)
        
    def get_text_products_by_type_and_location(self, typeID, locationID):
        """
        Returns a list of text products of a given type for a given issuance location.

        Args:
        - typeID (str): Text product type ID.
        - locationID (str): Location ID.
        
        Returns:
        - dict: Dictionary containing list of text product of a given type for a given issuance location.
        """
        endpoint = f"products/types/{typeID}/locations/{locationID}"
        return mr.make_request(self.base_url+endpoint)
        
    def get_zones(self):
        """
        Returns a list of zones.

        Args:
        - None

        Returns:
        - dict: A dictionary containing a list of zones.
        """
        endpoint = "zones"
        return mr.make_request(self.base_url+endpoint)

# HAVEN'T FIGURED THIS ONE OUT. PARAMETER IS A QUERY, NOT PATH  
    def get_zones_by_type(self, Type):
        """
        Returns a list of valid text product types for a given issuance location.

        Args:
        - Type (str): Zone type. Available values : land, marine, forecast, public, coastal, offshore, fire, county.
        
        Returns:
        - dict: Dictionary containing list of valid text product types for a given location.
        """
        endpoint = f"zones/{Type}"
        return mr.make_request(self.base_url+endpoint)

    def get_zone_metadata(self, Type, zoneID):
        """
        Returns metadata about a given zone.

        Args:
        - Type (str): Zone type. Available values : land, marine, forecast, public, coastal, offshore, fire, county.
        - zoneID (str): NWS public zone/county identifier.
        
        Returns:
        - dict: Dictionary containing metadata about a given zone.
        """
        endpoint = f"zones/{Type}/{zoneID}"
        return mr.make_request(self.base_url+endpoint)
        
    def get_zone_forecast(self, Type, zoneID):
        """
        Returns the current zone forecast for a given zone.

        Args:
        - Type (str): Zone type. Available values : land, marine, forecast, public, coastal, offshore, fire, county.
        - zoneID (str): NWS public zone/county identifier.
        
        Returns:
        - dict: Dictionary containing zone forecast for a given zone.
        """
        endpoint = f"zones/{Type}/{zoneID}/forecast"
        return mr.make_request(self.base_url+endpoint)
        
    def get_zone_observations(self, zoneID):
        """
        Returns a list of observations for a given zone.

        Args:
        - zoneID (str): NWS public zone/county identifier.
        
        Returns:
        - dict: Dictionary containing a list of observations for a given zkne.
        """
        endpoint = f"zones/forecast/{zoneID}/observations"
        return mr.make_request(self.base_url+endpoint)
        
    def get_observation_stations_by_zone(self, zoneID):
        """
        Returns a list of observation stations for a given zone.

        Args:
        - zoneID (str): NWS public zone/county identifier.
        
        Returns:
        - dict: Dictionary containing a list of observation stations for a given zkne.
        """
        endpoint = f"zones/forecast/{zoneID}/stations"
        return mr.make_request(self.base_url+endpoint)
    
    
    
    
  
       
       
        
        
     
      
      
      
        
    
    
      
      
      
   
   
   
   
        
 
      
      
      
        
        
        
    
       
     
     
     
        
    
       
        
        
   
        
        
    
    
    


from .. helperFuncs import make_request as mr

class DogAPI:
    """
    A class to interact with the Dog API.
    
    Attributes:
    - base_url: The base URL for the API.
    - about: A short description of the API.
    """
    def __init__(self):
        self.base_url = "https://dog.ceo/api/"
        self.about = "The Dog API returns URLs for dog images, either at random or by breed."
        
    def get_docs_url(self):
        """
        Returns the URL for the Dog API documentation.
        
        Args:
        - None
        
        Returns:
        - string: The URL for the Dog API documentation.
        """
        return "https://dog.ceo/dog-api/documentation/"
        
    def get_all_dog_breeds(self):
        """
        Returns a list of all the dog breeds in the Dog API.
        
        Args:
        - None
        
        Returns:
        - dict: Returns a dictionary containing a list of all the dog breeds in the Dog API.
        """
        endpoint = "breeds/list/all"
        return mr.make_request(self.base_url+endpoint)
        
    def get_random_dog_image(self):
        """
        Returns the URL for a random dog image.
        
        Args:
        - None
        
        Returns:
        - dict: Dictionary containing the URL for a random dog image.
        """
        endpoint = "breeds/image/random"
        return mr.make_request(self.base_url+endpoint)
        
    def get_dog_images_by_breed(self, breed):
        """
        Returns an array of URLs for images of dogs of the specified breed.
        
        Args:
        - breed (str): The breed of the dog.
        
        Returns:
        - dict: Dictionary containing a list of URLs of images for the specified dog breed.
        """
        endpoint = f"breed/{breed.lower()}/images"
        return mr.make_request(self.base_url+endpoint)
        
    def get_all_subBreeds(self, breed):
        """
        Returns a list of all sub-breeds for the specified dog breed.
        
        Args:
        - breed (str): The breed of the dog.
        
        Returns:
        - dict: Dictionary containing a list of all sub-breeds for the specified dog breed.
        """
        endpoint = f"breed/{breed.lower()}/list"
        return mr.make_request(self.base_url+endpoint)
        
    def get_random_dog_image_by_breed(self, breed):
        """
        Returns the URL for a random dog image of the specified breed.
        
        Args:
        - breed (str): The breed of the dog.
        
        Returns:
        - dict: Dictionary the URL for a random dog image of the specified breed.
        """
        endpoint = f"breed/{breed.lower()}/images/random"
        return mr.make_request(self.base_url+endpoint)
  
   
        
    


from .. helperFuncs import make_request as mr

class Zippopotomus:
    """
    A class to interact with the Zippopotomus API.
    
    Attributes:
    - base_url: The basee URL of the API.
    - about: A short description of the API.
    """
    def __init__(self):
        self.base_url = "http://api.zippopotam.us/"
        self.about = "Zippopotamus is an open source project that is focused on converting zip codes into valid geographical locations."
        
    def get_docs_url(self):
        """
        Returns the URL for the Zippopotomus API documentation.
        
        Args:
        - None
        
        Returns:
        - string: The URL for the API docs.
        """
        return "https://www.zippopotam.us/"
        
    def get_info_by_zipcode(self, country, zipcode):
        """
        Returns information on a given zipcode in a given country.
        
        Args:
        - country (str): The country code as defined in the API docs. See docs for supported countries. 
        - zipcode (int): The zipcode in question. See docs for range and format of zipcodes by country.
        
        Returns:
        - dict: A dictionary containing information about the given zipcode in the given country.
        """
        endpoint = f"{country}/{zipcode}"
        return mr.make_request(self.base_url+endpoint)
        
    def get_zipcode_by_city(self, country, state, city):
        """
        Returns the zipcode(s) of the city matching the parameters.
        
        Args:
        - country (str): The country code as defined in the API docs.
        - state (str): The state code.
        - city (str): The city in question.
        
        Returns:
        - dict: A dictionary containing the zipcode of the given location.
        """
        endpoint = f"{country}/{state}/{city}"
        return mr.make_request(self.base_url+endpoint)


nokey_apis = {
    "country_info": ["RestCountries"],
    "finance_and_crypto": ["Wallstreet Bets", "Exchange API", "Coinmap"],
    "geolocation": ["IP API", "Zippopotomus"],
    "spaceflight": ["Spaceflight News"],
    "education": ["University Domains and Names"],
    "food": ["Fruityvice"],
    "random": ["RandomUserGenerator"],
    "jokes": ["JokeAPI"],
    "animals": ["DogAPI"],
    "science_and_nature": ["Nobel Prize API", "Integrative Taxonomic Information System"],
    "weather": ["National Weather Service API"],
    "developer_tools": ["URL Shortener", "URLHaus", "APIsGuru", "Microlink", "FilterLists"],
    "inspiration": ["Dictum"],
    "language": ["Free Dictionary"],
    "games": ["Free To Game", "Open Trivia Database"],
    "activities": ["Bored API"],
    "calendar": ["Nager.Date"],
    "government": ["Federal Register"],
    "art_and_images": ["Lorem Picsum"],
    "tv_and_film": ["Star Trek API"],
    "books": ["Gutendex"]
}



from .. helperFuncs import make_request as mr

class Fruityvice:
    """
    A class to interact with the Fruityvice API.
    
    Attributes:
    - base_url: The base url of the Fruityvice API.
    - about: A short description of the API.
    """
    
    def __init__(self):
        self.base_url = "https://www.fruityvice.com/api/fruit/"
        self.about = "Fruityvice is an API that provides information on fruits and their nutritional value."
        
    def get_docs_url(self):
        """
        Returns the url for the Fruityvice API documentation.
        
        Args:
        - None
        
        Returns:
        - string: URL for API documentation.
        """
        return "https://www.fruityvice.com/doc/index.html"
        
    def get_all_fruit(self):
        """
        Returns all the fruit listed in the Fruityvice API.
        
        Args:
        - None
        
        Returns:
        - dict: Dictionary containing all the fruit listed in the API.
        """
        endpoint = "all"
        return mr.make_request(self.base_url+endpoint)
        
    def get_fruit_by_nutrition(self, nutrition, minVal=0, maxVal=1000):
        """
        Returns a fruit with its information based on a minimum and maximum range of values for the specified nutritional component.
        
        Args:
        - nutrition (str): A nutritional component. Possible values are carbohydrates, protein, fat, calories, and sugar.
        - minVal (float): The minimum value of the range specified for nutritional component (per 100g of the fruit). Defaults to its minimum possible value, which is 0.
        - maxVal (float): The maximum value of the range specified for nutritional component (per 100g of the fruit). Defaults to its maximum possible value, which is 1000.
        
        Returns:
        - dict: Dictionary containing information on the fruit matching the specified values.
        """
        endpoint = f"{nutrition}?min={minVal}&max={maxVal}"
        return mr.make_request(self.base_url+endpoint)
        
    def get_fruit_by_family(self, family):
        """
        Returns fruit matching the given family.
        
        Args:
        - family (str): The name of a fruit family.
        
        Returns:
        -dict: Dictionary containing fruit of the given family.
        """
        endpoint = f"family/{family}"
        return mr.make_request(self.base_url+endpoint)
        
    def get_fruit_by_genus(self, genus):
        """
        Returns fruit matching the given genus.
        
        Args:
        - genus (str): The name of a fruit genus.
        
        Returns:
        -dict: Dictionary containing fruit of the given genus.
        """
        endpoint = f"genus/{genus}"
        return mr.make_request(self.base_url+endpoint)
        
    def get_fruit_by_order(self, order):
        """
        Returns fruit matching the given order.
        
        Args:
        - order (str): The name of a fruit order.
        
        Returns:
        -dict: Dictionary containing fruit of the given order.
        """
        endpoint = f"order/{order}"
        return mr.make_request(self.base_url+endpoint)
     
     
        
    



from .. helperFuncs import make_request as mr

class ExchangeAPI:
    """
    A class for interacting with Exchange API.
    
    Attributes:
    - base_url: The base URL of the API.
    - about: A short description of the API.
    """
    def __init__(self):
        self.base_url = "https://cdn.jsdelivr.net/npm/@fawazahmed0/currency-api@"
        self.about = "ExchangeAPI is a free currency exchange rates API with 150+ currencies and no rate limits."
        
    def get_docs_url(self):
        """
        Returns the URL for the ExchangeAPI documentation.
        
        Args:
        - None
        
        Returns:
        - string: The URL for the API docs.
        """
        return "https://github.com/fawazahmed0/exchange-api"
        
    def get_available_currencies(self, minified_version=False):
        """
        Returns all the available currencies in the API.
        
        Args:
        - minified_version (bool): An optional parameter for getting a minified version of the currency list. Default is false.
        
        Returns:
        - dict: A dictionary containing a list of the available currencies.
        """
        if minified_version:
            endpoint = "latest/v1/currencies.min.json"
            print(endpoint)
            return mr.make_request(self.base_url+endpoint)
        else:
            endpoint = "latest/v1/currencies.json"
            return mr.make_request(self.base_url+endpoint)
            
    def get_rates_from_base(self, currency_code, date="latest", minified_version=False):
        """
        Returns exchange rates list for the given currency code on the given date.
        
        Args:
        - currency_code (str): The code for the base currency used to get exchange rates.
        - date (str): Date for the exchange rates. Default is latest, but other values must be in YYYY-MM-DD format.
        - minified_version (bool): An optional parameter for getting a minified version of the currency list. Default is false.
        
        Returns:
        - dict: A dictionary containing a list of exchange rates.
        """
        if minified_version:
            endpoint = f"{date}/v1/currencies/{currency_code.lower()}.min.json"
            print(endpoint)
            return mr.make_request(self.base_url+endpoint)
        else:
            endpoint = f"{date}/v1/currencies/{currency_code.lower()}.json"
            return mr.make_request(self.base_url+endpoint)
            
    
            
    
        
        
        


from .. helperFuncs import make_request as mr

class UrlShortener:
    """
    A class for interacting with the URL Shortener API.
    
    Attributes:
    - base_url: The base URL of the URL Shortener API.
    - about: A short description of the API.
    """
    def __init__(self):
        self.base_url = "https://is.gd/create.php?"
        self.about = "This URL Shortener API (from is.gd) is a URL shortener service."
        
    def get_docs_url(self):
        """
        Returns the URL for the URL Shortener API documentation.
        
        Args:
        - None
        
        Returns:
        - string: The URL for the API docs
        """
        return "https://is.gd/apishorteningreference.php?"
        
    def shorten_url(self, url, shorturl=None):
        """
        Returns the shortened URL for the given parameters.
        
        Args:
        - url (str): The original URL to be shortened.
        - shorturl (str): Optional parameter for a desired short URL if available (case sensitive). Must be between 5 and 30 characters. Defaults to None.
        
        Returns:
        - dict: Dictionary containing the shortened URL.
        """
        if shorturl is not None:
            if len(shorturl) < 31 and len(shorturl) > 4:
                endpoint = f"format=json&url={url}&shorturl={shorturl}"
            else:
                print("Chosen shortened URL must be between 5 and 30 characters")
                return ""
        else:
            endpoint = f"format=json&url={url}"
            return mr.make_request(self.base_url+endpoint)
                    
                    


from .. helperFuncs import make_request as mr

class FreeDictionary:
    """
    A class to interact with the Free Dictionary API.
    
    Attributes:
    - base_url: The base URL of the Free Dictionary API.
    - about: A short description of the API.
    """
    def __init__(self):
        self.base_url = "https://api.dictionaryapi.dev/api/v2/entries/en/"
        self.about = "The Free Dictionary API is a powerful tool that allows you to access the vast array of dictionary data."
        
    def get_docs_url(self):
        """
        Returns the URL for the Free Dictionary API documentation.
        
        Args:
        - None
        
        Returns:
        - string: The URL for thre API documentation.
        """
        return "https://dictionaryapi.dev/"
        
    def look_up_word(self, word):
        """
        Returns the definition and other information of a word.
        
        Args:
        - word (str): The word to be looked up.
        
        Returns:
        - dict: A dictionary containing the definition of a word and other related information.
        """
        endpoint = f"{word}"
        return mr.make_request(self.base_url+endpoint)


from .. helperFuncs import make_request as mr

class FederalRegister:
    """
    A class for interacting with the Federal Register API.
    
    Attributes:
    base_url: The base URL of the API.
    about: A short description of the API.
    """
    def __init__(self):
        self.base_url = "https://www.federalregister.gov/api/v1/"
        self.about = "FederalRegister.gov provides multiple public API endpoints. These can be used to access information in the Federal Register, the daily journal of the US government."
   
    def get_docs_url(self):
        """
        Returns the URL for the Federal Register API documentation.
        
        Args:
        - None
        
        Returns:
        - string: The URL for the API docs.
        """
        return "https://www.federalregister.gov/developers/documentation/api/v1"
        
    def get_fed_reg_document(self, document_num):
        """
        Returns a Federal Register document of the given number.
        
        Args:
        - document_num (str): The unique number for the document.
        
        Returns:
        - dict: A dictionary containing the document matching the given number.
        """
        endpoint = f"documents/{document_num}.json"
        return mr.make_request(self.base_url+endpoint)
        
    def get_multiple_fed_reg_documents(self, document_nums):
        """
        Returns multiple Federal Register documents of the given numbers.
        
        Args:
        - document_nums (str): A string of unspaced, comma-separated document numbers.
        
        Returns:
        - dict: A dictionary containing the documents matching the given numbers.
        """
        endpoint = f"documents/{document_nums}.json"
        return mr.make_request(self.base_url+endpoint)
        
    def search_all_documents(self, search_term, per_page=20, doc_type=None):
        """
        Returns the Federal Register documents published since 1994 matching the given parameters.
        
        Args:
        - search_term (str): The term by which to search the documents.
        - per_page (int): The number of documents to return at once; default is 20; 1000 maximum.
        - doc_type (str): Additional optional search by document type. Supported values are rule (final rule), prorule (proposed rule), notice (notice), and presdocu (presidential document)
        
        Returns:
        - dict: A dictionary containing the documents published since 1994 matching the given values.
        """
        if doc_type is not None:
            endpoint = f"documents.json?per_page={per_page}&conditions[term]={search_term}&conditions[type][]={doc_type.upper()}"
            return mr.make_request(self.base_url+endpoint)
        else:            
            endpoint = f"documents.json?per_page={per_page}&conditions[term]={search_term}"
            return mr.make_request(self.base_url+endpoint)
        
    def get_document_counts_by_facet(self, facet, search_term=None, doc_type=None):
        """
        Returns counts of matching Federal Register documents grouped by a facet.
        
        Args:
        - facet (str): The facet by which to group the documents. Supported values are daily, weekly, monthly, quarterly, yearly, agency, topic, section, type, and subtype.
        - search_term (str): Option search query term. Defaults to None.
        - doc_type (str): Additional optional search by document type. Supported values are rule (final rule), prorule (proposed rule), notice (notice), and presdocu (presidential document)
        
        Returns:
        - dict: Returns a dictionary of document counts for the given parameters.
        """
        if search_term is not None and doc_type is None:
            endpoint = f"documents/facets/{facet}?conditions[term]={search_term}"
            return mr.make_request(self.base_url+endpoint)
        elif doc_type is not None and search_term is None:
            endpoint = f"documents/facets/{facet}?conditions[type][]={doc_type.upper()}"
            return mr.make_request(self.base_url+endpoint)
        else:
            endpoint = f"documents/facets/{facet}"
            return mr.make_request(self.base_url+endpoint)
            
    def get_document_toc_by_date(self, pub_date):
        """
        Returns the document table of contents based on print edition.
        
        Args:
        - pub_date (str): The exact publication date (YYYY-MM-DD).
        
        Returns:
        - dict: A dictionary containing the document table of contents that matches the given date.
        """
        endpoint = f"issues/{pub_date}.json"
        return mr.make_request(self.base_url+endpoint)
        
    def get_public_inspection_document(self, document_num):
        """
        Returns a single public inspection document matching the given number.
        
        Args:
        - document_num (str): The number matching the public inspection document.
        
        Returns:
        - dict: A dictionary containing a single public inspection document matching the given number.
        """
        endpoint = f"public-inspection-documents/{document_num}.json"
        return mr.make_request(self.base_url+endpoint)
        
    def get_multiple_public_inspection_documents(self, document_nums):
        """
        Returns the public inspection documents matching the given numbers.
        
        Args:
        - document_nums (str): The numbers matching the public inspection documents, separated by commas, no spaces.
        
        Returns:
        - dict: A dictionary containing the public inspection documents matching the given numbers.
        """
        endpoint = f"public-inspection-documents/{document_nums}.json"
        return mr.make_request(self.base_url+endpoint)
        
    def get_current_public_inspection_documents(self):
        """
        Returns all the public inspection documents that are currently on public inspection.
        
        Args:
        - None
        
        Returns:
        - dict: A dictionary containing all the public inspection documents that are currently on public inspection.
        """
        endpoint = "public-inspection-documents/current.json"
        return mr.make_request(self.base_url+endpoint)
        
    def search_all_public_inspection_documents(self, pub_date, per_page=20, search_term=None, doc_type=None):
        """
        Returns the Federal Register documents published since 1994 matching the given parameters.
        
        Args:
        - pub_date (str): Public inspection issue date (YYYY-MM-DD).
        - per_page (int): The number of documents to return at once; default is 20; 1000 maximum.
        - search_term (str): The term by which to search the documents.
        - doc_type (str): Additional optional search by document type. Supported values are rule (final rule), prorule (proposed rule), notice (notice), and presdocu (presidential document)
        
        Returns:
        - dict: A dictionary containing the documents published since 1994 matching the given values.
        """
        if search_term is not None and doc_type is None:
            endpoint = f"public-inspection-documents.json?per_page={per_page}&conditions[available_on]={pub_date}&conditions[term]={search_term}"
            return mr.make_request(self.base_url+endpoint)
        elif doc_type is not None and search_term is None:
            endpoint = f"public-inspection-documents.json?per_page={per_page}&conditions[available_on]={pub_date}&conditions[type][]={doc_type.upper()}"
            return mr.make_request(self.base_url+endpoint)
        elif search_term is not None and doc_type is not None:
            endpoint = f"public-inspection-documents.json?per_page={per_page}&conditions[available_on]={pub_date}&conditions[term]={search_term}&conditions[type][]={doc_type.upper()}"
            print(self.base_url+endpoint)
            return mr.make_request(self.base_url+endpoint)
        else:            
            endpoint = f"public-inspection-documents.json?per_page={per_page}&conditions[available_on]={pub_date}"
            return mr.make_request(self.base_url+endpoint)
            
    def get_agencies_details(self):
        """
        Returns all agency details.
        
        Args:
        - None
        
        Returns:
        - dict: A dictionary containing all agency details.
        """
        endpoint = "agencies"
        return mr.make_request(self.base_url+endpoint)
        
    def get_agency_by_slug(self, slug):
        """
        Returns a particular agency's details.
        
        Args:
        - slug (str): The Federal Register slug for the agency. See Federal Register API docs for extensive list of possible values.
        
        Returns:
        - dict: A dictionary containing a particular agency's details.
        """
        endpoint = f"agencies/{slug}"
        return mr.make_request(self.base_url+endpoint)
        
    def get_image_variants_by_id(self, image_id):
        """
        Returns the available image variants and their metadata for a single image identifier.
        
        Args:
        - image_id (str): The Federal Register image identifier.
        
        Returns:
        - dict: A dictionary containing the available image variants and their metadata for a single image identifier.
        """
        endpoint = f"images/{image_id}"
        return mr.make_request(self.base_url+endpoint)
        
    def get_suggested_searches_by_section(self, section):
        """
        Returns all suggested searches or limit by FederalRegister.gov section.
        
        Args:
        - section (str): The Federal Register slug for the section. Supported values are business-and-industry, environment, health-and-public-welfare, money, science-and-technology, and world.
        
        Returns:
        - dict: A dictionary containing all the suggested searches or limit by Federal.Register.gov section.
        """
        endpoint = f"suggested_searches?conditions[sections]={section}"
        return mr.make_request(self.base_url+endpoint)
        
    def get_suggested_search_by_slug(self, slug):
        """
        Returns a particular suggested search.
        
        Args:
        - slug (str): The Federal Register slug for the suggested search. See Federal Register API docs for extensive list of possible values.
        
        Returns:
        - dict: A dictionary containing a particular suggested search.
        """
        endpoint = f"suggested_searches/{slug}"
        return mr.make_request(self.base_url+endpoint)
 
  
   
   
        
        
 
 
 
 
        
    
 
       


import os

import requests

from .. helperFuncs import make_request as mr


class URLHaus:
    """
    A class to interact with the URLHaus API.
    
    Attributes:
    - base_url: The base URL of the API.
    - about: A short description of the API.
    """
    def __init__(self):
        self.base_url = "https://urlhaus-api.abuse.ch/v1/"
        self.about = "URLhaus is a project operated by abuse.ch. The purpose of the project is to collect, track and share malware URLs, helping network administrators and security analysts to protect their network and customers from cyber threats."
        
    def get_docs_url(self):
        """
        Returns the URL for the URLHaus API documentation.
        
        Args:
        - None
        
        Returns:
        - string: The URL for the API docs.
        """
        return "https://urlhaus-api.abuse.ch/"
        
    def get_recent_urls(self, limit=None):
        """
        Returns a list of recent URLs added to URLHaus.
        
        Args:
        - limit (int): Optional limit of URLs returned. Defaults to None.
        
        Returns:
        - dict: A dictionary containing a list of recent URLs added to URLHaus.
        """
        if limit is not None:
            endpoint = f"urls/recent/limit/{limit}"
        else:
            endpoint = "urls/recent"
        return mr.make_request(self.base_url+endpoint)
        
    def get_recent_payloads(self, limit=None):
        """
        Returns a list of recent payloads seen by URLHaus.
        
        Args:
        - limit (int): Optional limit of payloads returned. Defaults to None.
        
        Returns:
        - dict: A dictionary containing a list of recent payloads seen by URLHaus.
        """
        if limit is not None:
            endpoint = f"payloads/recent/limit/{limit}"
        else:
            endpoint = "payloads/recent"
        return mr.make_request(self.base_url+endpoint)
        
    def get_info_about_url(self, url):
        """
        Returns any information URLHaus may have about a specific URL.
        
        Args:
        - url (str): The URL in question.
        
        Returns:
        - dict: A dictionary containing information about a specific URL.
        """
        payload = {"url": f"{url}"}
        endpoint = "url/"
        response = requests.post(self.base_url+endpoint, payload)
        if response.status_code == 200:
            return response.json()
        else:
            return f"Error: {response.status_code}"
            
    def get_info_about_url_by_id(self, urlID):
        """
        Returns any information URLHaus may have about a specific URL.
        
        Args:
        - urlID (int): The id of the URL in question.
        
        Returns:
        - dict: A dictionary containing information about a specific URL.
        """
        payload = {"urlid": f"{urlID}"}
        endpoint = "urlid/"
        response = requests.post(self.base_url+endpoint, payload)
        if response.status_code == 200:
            return response.json()
        else:
            return f"Error: {response.status_code}"
            
    def get_info_about_host(self, host):
        """
        Returns any information URLHaus may have about a specific host.
        
        Args:
        - host (str): The IPv4 address, domain name, or hostname in question.
        
        Returns:
        - dict: A dictionary containing information about a specific host.
        """
        payload = {"host": f"{host}"}
        endpoint = "host/"
        response = requests.post(self.base_url+endpoint, payload)
        if response.status_code == 200:
            return response.json()
        else:
            return f"Error: {response.status_code}"
            
    def get_info_about_payload(self, p_load):
        """
        Returns any information URLHaus has about a specific payload (malware sample).
        
        Args:
        - p_load (str): The MD5 hash or SHA256 hash of the payload (malware sample).
        
        Returns:
        - dict: A dictionary containing information about a specific payload.
        """
        payload = {"payload": f"{p_load}"}
        endpoint = "payload/"
        response = requests.post(self.base_url+endpoint, payload)
        if response.status_code == 200:
            return response.json()
        else:
            return f"Error: {response.status_code}"
            
    def get_info_about_tag(self, tag):
        """
        Returns any information URLHaus has about a specific tag.
        
        Args:
        - tag (str): The tag to query.
        
        Returns:
        - dict: A dictionary containing information about a specific tag.
        """
        payload = {"tag": f"{tag}"}
        endpoint = "tag/"
        response = requests.post(self.base_url+endpoint, payload)
        if response.status_code == 200:
            return response.json()
        else:
            return f"Error: {response.status_code}"
            
    def get_info_about_signature(self, signature):
        """
        Returns any information URLHaus has about a specific signature.
        
        Args:
        - signature (str): The signature to query.
        
        Returns:
        - dict: A dictionary containing information about a specific signature.
        """
        payload = {"signature": f"{signature}"}
        endpoint = "signature/"
        response = requests.post(self.base_url+endpoint, payload)
        if response.status_code == 200:
            return response.json()
        else:
            return f"Error: {response.status_code}"
            
    def download_malware_sample(self, sha256):
        """
        Downloads a zip file containing the malware sample (payload).
    
        Args:
        - base_url (str): The base URL of the API.
        - sha256 (str): The SHA256 hash identifying the malware sample (payload) to be downloaded.
    
        Returns:
        - str: Path to the downloaded ZIP file if successful, error message otherwise.
        """
        # Define the endpoint and construct the URL
        endpoint = f"download/{sha256}"
        url = self.base_url + endpoint

        # Send the GET request
        response = requests.get(url)

        # Check if the request was successful (status code 200)
        if response.status_code == 200:
            # Check if the response contains a ZIP file
            if response.headers.get('content-disposition'):
                # Extract the filename from the content-disposition header
                filename = response.headers['content-disposition'].split('=')[1]
                # Get the current directory
                current_directory = os.getcwd()
                # Define the path to save the ZIP file
                zip_path = os.path.join(current_directory, filename)
                # Save the content to the ZIP file
                with open(zip_path, 'wb') as f:
                    f.write(response.content)
                return zip_path
            else:
                return "Error: Response does not contain a ZIP file."
        elif response.status_code == 404:
            return "Error: Malware sample not found."
        else:
            return f"Error: {response.status_code}"      
     
     
     
     
       
        
    
    


from .. helperFuncs import make_request as mr

class STAPI:
    """
    A class for interacting with the Star Trek API.
    
    Attributes:
    - base_url: The base URL of the API.
    - about: A short description of the API.
    """
    def __init__(self):
        self.base_url = "https://stapi.co/api/"
        self.about = "STAPI (Star Trek API) is an API for accessing information about all things Star Trek."
        
    def get_docs_url(self):
        """
        Returns the URL for the Star Trek API.
        
        Args:
        - None
        
        Returns:
        - str: The URL for the API.
        """
        return "https://editor.swagger.io/?url=https://stapi.co/api/v1/rest/common/download/stapi.yaml"
        
    def get_animal_by_id(self, uid):
        """
        Returns information about a Star Trek animal matching the unique id.
        
        Args:
        - uid (str): The unique identifier for the animal.
        
        Returns:
        - dict: A dictionary containing information about the animal.
        """
        endpoint = f"v1/rest/animal?uid={uid}"
        return mr.make_request(self.base_url+endpoint)
        
    def get_animals(self, pageNumber=0, pageSize=50):
        """
        Returns a list of animals of the specified page number and page size.
        
        Args:
        - pageNumber (int): The page number of the list. Defaults to 0, the first page.
        - pageSize (int): The number of animals per page. Defaults to 50.
        
        Returns:
        - dict: A dictionary containing a list of animals.
        """
        endpoint = f"v1/rest/animal/search?pageNumber={pageNumber}&pageSize={pageSize}"
        return mr.make_request(self.base_url+endpoint)
        
    def get_astronomical_object_by_id(self, uid):
        """
        Returns information about a Star Trek astronomical object matching the unique id.
        
        Args:
        - uid (str): The unique identifier for the astronomical object.
        
        Returns:
        - dict: A dictionary containing information about the astronomical object.
        """
        endpoint = f"v2/rest/astronomicalObject?uid={uid}"
        return mr.make_request(self.base_url+endpoint)
        
    def get_astronomical_objects(self, pageNumber=0, pageSize=50):
        """
        Returns a list of astronomical objects of the specified page number and page size.
        
        Args:
        - pageNumber (int): The page number of the list. Defaults to 0, the first page.
        - pageSize (int): The number of objects per page. Defaults to 50.
        
        Returns:
        - dict: A dictionary containing a list of astronomical objects.
        """
        endpoint = f"v2/rest/astronomicalObject/search?pageNumber={pageNumber}&pageSize={pageSize}"
        return mr.make_request(self.base_url+endpoint)
        
    def get_book_by_id(self, uid):
        """
        Returns information about a Star Trek book matching the unique id.
        
        Args:
        - uid (str): The unique identifier for the book.
        
        Returns:
        - dict: A dictionary containing information about the book.
        """
        endpoint = f"v2/rest/book?uid={uid}"
        return mr.make_request(self.base_url+endpoint)
        
    def get_books(self, pageNumber=0, pageSize=50):
        """
        Returns a list of Star Trek books of the specified page number and page size.
        
        Args:
        - pageNumber (int): The page number of the list. Defaults to 0, the first page.
        - pageSize (int): The number of books per page. Defaults to 50.
        
        Returns:
        - dict: A dictionary containing a list of books.
        """
        endpoint = f"v2/rest/book/search?pageNumber={pageNumber}&pageSize={pageSize}"
        return mr.make_request(self.base_url+endpoint)
        
    def get_book_collection_by_id(self, uid):
        """
        Returns information about a Star Trek book collection matching the unique id.
        
        Args:
        - uid (str): The unique identifier for the book collection.
        
        Returns:
        - dict: A dictionary containing information about the book collection.
        """
        endpoint = f"v1/rest/bookCollection?uid={uid}"
        return mr.make_request(self.base_url+endpoint)
        
    def get_book_collections(self, pageNumber=0, pageSize=50):
        """
        Returns a list of Star Trek book collections of the specified page number and page size.
        
        Args:
        - pageNumber (int): The page number of the list. Defaults to 0, the first page.
        - pageSize (int): The number of book collections per page. Defaults to 50.
        
        Returns:
        - dict: A dictionary containing a list of book collections.
        """
        endpoint = f"v1/rest/bookCollection/search?pageNumber={pageNumber}&pageSize={pageSize}"
        return mr.make_request(self.base_url+endpoint)
        
    def get_book_series_by_id(self, uid):
        """
        Returns information about a Star Trek book series matching the unique id.
        
        Args:
        - uid (str): The unique identifier for the book series.
        
        Returns:
        - dict: A dictionary containing information about the book series.
        """
        endpoint = f"v1/rest/bookSeries?uid={uid}"
        return mr.make_request(self.base_url+endpoint)
        
    def get_book_series(self, pageNumber=0, pageSize=50):
        """
        Returns a list of Star Trek book series of the specified page number and page size.
        
        Args:
        - pageNumber (int): The page number of the list. Defaults to 0, the first page.
        - pageSize (int): The number of book series per page. Defaults to 50.
        
        Returns:
        - dict: A dictionary containing a list of book series.
        """
        endpoint = f"v1/rest/bookSeries/search?pageNumber={pageNumber}&pageSize={pageSize}"
        return mr.make_request(self.base_url+endpoint)
        
    def get_character_by_id(self, uid):
        """
        Returns information about a Star Trek character matching the unique id.
        
        Args:
        - uid (str): The unique identifier for the character.
        
        Returns:
        - dict: A dictionary containing information about the character.
        """
        endpoint = f"v1/rest/character?uid={uid}"
        return mr.make_request(self.base_url+endpoint)
        
    def get_characters(self, pageNumber=0, pageSize=50):
        """
        Returns a list of Star Trek characters of the specified page number and page size.
        
        Args:
        - pageNumber (int): The page number of the list. Defaults to 0, the first page.
        - pageSize (int): The number of characters per page. Defaults to 50.
        
        Returns:
        - dict: A dictionary containing a list of characters.
        """
        endpoint = f"v1/rest/character/search?pageNumber={pageNumber}&pageSize={pageSize}"
        return mr.make_request(self.base_url+endpoint)
        
    def get_comic_by_id(self, uid):
        """
        Returns information about a Star Trek comic matching the unique id.
        
        Args:
        - uid (str): The unique identifier for the comic.
        
        Returns:
        - dict: A dictionary containing information about the comic.
        """
        endpoint = f"v1/rest/comics?uid={uid}"
        return mr.make_request(self.base_url+endpoint)
        
    def get_comics(self, pageNumber=0, pageSize=50):
        """
        Returns a list of Star Trek comics of the specified page number and page size.
        
        Args:
        - pageNumber (int): The page number of the list. Defaults to 0, the first page.
        - pageSize (int): The number of comics per page. Defaults to 50.
        
        Returns:
        - dict: A dictionary containing a list of comics.
        """
        endpoint = f"v1/rest/comics/search?pageNumber={pageNumber}&pageSize={pageSize}"
        return mr.make_request(self.base_url+endpoint)
        
    def get_comic_collection_by_id(self, uid):
        """
        Returns information about a Star Trek comic collection matching the unique id.
        
        Args:
        - uid (str): The unique identifier for the comic collection.
        
        Returns:
        - dict: A dictionary containing information about the comic collection.
        """
        endpoint = f"v1/rest/comicCollection?uid={uid}"
        return mr.make_request(self.base_url+endpoint)
        
    def get_comic_collections(self, pageNumber=0, pageSize=50):
        """
        Returns a list of Star Trek comic collections of the specified page number and page size.
        
        Args:
        - pageNumber (int): The page number of the list. Defaults to 0, the first page.
        - pageSize (int): The number of comic collections per page. Defaults to 50.
        
        Returns:
        - dict: A dictionary containing a list of comic collections.
        """
        endpoint = f"v1/rest/comicCollection/search?pageNumber={pageNumber}&pageSize={pageSize}"
        return mr.make_request(self.base_url+endpoint)
        
    def get_comic_series_by_id(self, uid):
        """
        Returns information about a Star Trek comic series matching the unique id.
        
        Args:
        - uid (str): The unique identifier for the comic series.
        
        Returns:
        - dict: A dictionary containing information about the comic series.
        """
        endpoint = f"v1/rest/comicSeries?uid={uid}"
        return mr.make_request(self.base_url+endpoint)
        
    def get_comic_series(self, pageNumber=0, pageSize=50):
        """
        Returns a list of Star Trek comic series of the specified page number and page size.
        
        Args:
        - pageNumber (int): The page number of the list. Defaults to 0, the first page.
        - pageSize (int): The number of comic series per page. Defaults to 50.
        
        Returns:
        - dict: A dictionary containing a list of comic series.
        """
        endpoint = f"v1/rest/comicSeries/search?pageNumber={pageNumber}&pageSize={pageSize}"
        return mr.make_request(self.base_url+endpoint)
        
    def get_comic_strip_by_id(self, uid):
        """
        Returns information about a Star Trek comic strip matching the unique id.
        
        Args:
        - uid (str): The unique identifier for the comic strip.
        
        Returns:
        - dict: A dictionary containing information about the comic strip.
        """
        endpoint = f"v1/rest/comicStrip?uid={uid}"
        return mr.make_request(self.base_url+endpoint)
        
    def get_comic_strips(self, pageNumber=0, pageSize=50):
        """
        Returns a list of Star Trek comic strips of the specified page number and page size.
        
        Args:
        - pageNumber (int): The page number of the list. Defaults to 0, the first page.
        - pageSize (int): The number of comic strips per page. Defaults to 50.
        
        Returns:
        - dict: A dictionary containing a list of comic strips.
        """
        endpoint = f"v1/rest/comicStrip/search?pageNumber={pageNumber}&pageSize={pageSize}"
        return mr.make_request(self.base_url+endpoint)
        
    def get_company_by_id(self, uid):
        """
        Returns information about a Star Trek company matching the unique id.
        
        Args:
        - uid (str): The unique identifier for the company.
        
        Returns:
        - dict: A dictionary containing information about the company.
        """
        endpoint = f"v2/rest/company?uid={uid}"
        return mr.make_request(self.base_url+endpoint)
        
    def get_companies(self, pageNumber=0, pageSize=50):
        """
        Returns a list of Star Trek companies of the specified page number and page size.
        
        Args:
        - pageNumber (int): The page number of the list. Defaults to 0, the first page.
        - pageSize (int): The number of companies per page. Defaults to 50.
        
        Returns:
        - dict: A dictionary containing a list of companies.
        """
        endpoint = f"v2/rest/company/search?pageNumber={pageNumber}&pageSize={pageSize}"
        return mr.make_request(self.base_url+endpoint)
        
    def get_conflict_by_id(self, uid):
        """
        Returns information about a Star Trek conflict matching the unique id.
        
        Args:
        - uid (str): The unique identifier for the conflict.
        
        Returns:
        - dict: A dictionary containing information about the conflict.
        """
        endpoint = f"v2/rest/conflict?uid={uid}"
        return mr.make_request(self.base_url+endpoint)
        
    def get_conflicts(self, pageNumber=0, pageSize=50):
        """
        Returns a list of Star Trek conflicts of the specified page number and page size.
        
        Args:
        - pageNumber (int): The page number of the list. Defaults to 0, the first page.
        - pageSize (int): The number of conflicts per page. Defaults to 50.
        
        Returns:
        - dict: A dictionary containing a list of conflicts.
        """
        endpoint = f"v1/rest/conflict/search?pageNumber={pageNumber}&pageSize={pageSize}"
        return mr.make_request(self.base_url+endpoint)
        
    def get_data_version(self):
        """
        Returns the data version of the API.
        
        Args:
        - None
        
        Returns:
        - dict: A dictionary containing the data version of the API.
        """
        endpoint = "v1/rest/common/dataVersion"
        return mr.make_request(self.base_url+endpoint)
        
    def get_element_by_id(self, uid):
        """
        Returns information about a Star Trek element matching the unique id.
        
        Args:
        - uid (str): The unique identifier for the element.
        
        Returns:
        - dict: A dictionary containing information about the element.
        """
        endpoint = f"v2/rest/element?uid={uid}"
        return mr.make_request(self.base_url+endpoint)
        
    def get_elements(self, pageNumber=0, pageSize=50):
        """
        Returns a list of Star Trek elements of the specified page number and page size.
        
        Args:
        - pageNumber (int): The page number of the list. Defaults to 0, the first page.
        - pageSize (int): The number of elements per page. Defaults to 50.
        
        Returns:
        - dict: A dictionary containing a list of elements.
        """
        endpoint = f"v2/rest/element/search?pageNumber={pageNumber}&pageSize={pageSize}"
        return mr.make_request(self.base_url+endpoint)
        
    def get_episode_by_id(self, uid):
        """
        Returns information about a Star Trek episode matching the unique id.
        
        Args:
        - uid (str): The unique identifier for the episode.
        
        Returns:
        - dict: A dictionary containing information about the episode.
        """
        endpoint = f"v1/rest/episode?uid={uid}"
        return mr.make_request(self.base_url+endpoint)
        
    def get_episodes(self, pageNumber=0, pageSize=50):
        """
        Returns a list of Star Trek episodes of the specified page number and page size.
        
        Args:
        - pageNumber (int): The page number of the list. Defaults to 0, the first page.
        - pageSize (int): The number of eposodes per page. Defaults to 50.
        
        Returns:
        - dict: A dictionary containing a list of episodes.
        """
        endpoint = f"v1/rest/episode/search?pageNumber={pageNumber}&pageSize={pageSize}"
        return mr.make_request(self.base_url+endpoint)
        
    def get_food_by_id(self, uid):
        """
        Returns information about a Star Trek food matching the unique id.
        
        Args:
        - uid (str): The unique identifier for the food.
        
        Returns:
        - dict: A dictionary containing information about the food.
        """
        endpoint = f"v1/rest/food?uid={uid}"
        return mr.make_request(self.base_url+endpoint)
        
    def get_foods(self, pageNumber=0, pageSize=50):
        """
        Returns a list of Star Trek foods of the specified page number and page size.
        
        Args:
        - pageNumber (int): The page number of the list. Defaults to 0, the first page.
        - pageSize (int): The number of foods per page. Defaults to 50.
        
        Returns:
        - dict: A dictionary containing a list of foods.
        """
        endpoint = f"v1/rest/food/search?pageNumber={pageNumber}&pageSize={pageSize}"
        return mr.make_request(self.base_url+endpoint)
        
    def get_literature_by_id(self, uid):
        """
        Returns information about Star Trek literature matching the unique id.
        
        Args:
        - uid (str): The unique identifier for the literature.
        
        Returns:
        - dict: A dictionary containing information about the literature.
        """
        endpoint = f"v1/rest/literature?uid={uid}"
        return mr.make_request(self.base_url+endpoint)
        
    def get_literature(self, pageNumber=0, pageSize=50):
        """
        Returns a list of Star Trek literature of the specified page number and page size.
        
        Args:
        - pageNumber (int): The page number of the list. Defaults to 0, the first page.
        - pageSize (int): The number of literature per page. Defaults to 50.
        
        Returns:
        - dict: A dictionary containing a list of literature.
        """
        endpoint = f"v1/rest/literature/search?pageNumber={pageNumber}&pageSize={pageSize}"
        return mr.make_request(self.base_url+endpoint)
        
    def get_location_by_id(self, uid):
        """
        Returns information about a Star Trek location matching the unique id.
        
        Args:
        - uid (str): The unique identifier for the location.
        
        Returns:
        - dict: A dictionary containing information about the location.
        """
        endpoint = f"v2/rest/location?uid={uid}"
        return mr.make_request(self.base_url+endpoint)
        
    def get_locations(self, pageNumber=0, pageSize=50):
        """
        Returns a list of Star Trek locations of the specified page number and page size.
        
        Args:
        - pageNumber (int): The page number of the list. Defaults to 0, the first page.
        - pageSize (int): The number of locations per page. Defaults to 50.
        
        Returns:
        - dict: A dictionary containing a list of locations.
        """
        endpoint = f"v2/rest/location/search?pageNumber={pageNumber}&pageSize={pageSize}"
        return mr.make_request(self.base_url+endpoint)
        
    def get_magazine_by_id(self, uid):
        """
        Returns information about a Star Trek magazine matching the unique id.
        
        Args:
        - uid (str): The unique identifier for the magazine.
        
        Returns:
        - dict: A dictionary containing information about the magazine.
        """
        endpoint = f"v1/rest/magazine?uid={uid}"
        return mr.make_request(self.base_url+endpoint)
        
    def get_magazines(self, pageNumber=0, pageSize=50):
        """
        Returns a list of Star Trek magazines of the specified page number and page size.
        
        Args:
        - pageNumber (int): The page number of the list. Defaults to 0, the first page.
        - pageSize (int): The number of magazines per page. Defaults to 50.
        
        Returns:
        - dict: A dictionary containing a list of magazines.
        """
        endpoint = f"v1/rest/magazine/search?pageNumber={pageNumber}&pageSize={pageSize}"
        return mr.make_request(self.base_url+endpoint)
        
    def get_magazine_series_by_id(self, uid):
        """
        Returns information about a Star Trek magazine series matching the unique id.
        
        Args:
        - uid (str): The unique identifier for the magazine series.
        
        Returns:
        - dict: A dictionary containing information about the magazine series.
        """
        endpoint = f"v1/rest/magazineSeries?uid={uid}"
        return mr.make_request(self.base_url+endpoint)
        
    def get_magazine_series(self, pageNumber=0, pageSize=50):
        """
        Returns a list of Star Trek magazine series of the specified page number and page size.
        
        Args:
        - pageNumber (int): The page number of the list. Defaults to 0, the first page.
        - pageSize (int): The number of magazine series per page. Defaults to 50.
        
        Returns:
        - dict: A dictionary containing a list of magazine series.
        """
        endpoint = f"v1/rest/magazineSeries/search?pageNumber={pageNumber}&pageSize={pageSize}"
        return mr.make_request(self.base_url+endpoint)
        
    def get_material_by_id(self, uid):
        """
        Returns information about a Star Trek material matching the unique id.
        
        Args:
        - uid (str): The unique identifier for the material.
        
        Returns:
        - dict: A dictionary containing information about the material.
        """
        endpoint = f"v1/rest/material?uid={uid}"
        return mr.make_request(self.base_url+endpoint)
        
    def get_materials(self, pageNumber=0, pageSize=50):
        """
        Returns a list of Star Trek materials of the specified page number and page size.
        
        Args:
        - pageNumber (int): The page number of the list. Defaults to 0, the first page.
        - pageSize (int): The number of materials per page. Defaults to 50.
        
        Returns:
        - dict: A dictionary containing a list of materials.
        """
        endpoint = f"v1/rest/material/search?pageNumber={pageNumber}&pageSize={pageSize}"
        return mr.make_request(self.base_url+endpoint)
        
    def get_medical_condition_by_id(self, uid):
        """
        Returns information about a Star Trek medical condition matching the unique id.
        
        Args:
        - uid (str): The unique identifier for the medical condition.
        
        Returns:
        - dict: A dictionary containing information about the medical condition.
        """
        endpoint = f"v1/rest/medicalCondition?uid={uid}"
        return mr.make_request(self.base_url+endpoint)
        
    def get_medical_conditions(self, pageNumber=0, pageSize=50):
        """
        Returns a list of Star Trek medical conditions of the specified page number and page size.
        
        Args:
        - pageNumber (int): The page number of the list. Defaults to 0, the first page.
        - pageSize (int): The number of medical conditions per page. Defaults to 50.
        
        Returns:
        - dict: A dictionary containing a list of medical conditions.
        """
        endpoint = f"v1/rest/medicalCondition/search?pageNumber={pageNumber}&pageSize={pageSize}"
        return mr.make_request(self.base_url+endpoint)
        
    def get_movie_by_id(self, uid):
        """
        Returns information about a Star Trek movie matching the unique id.
        
        Args:
        - uid (str): The unique identifier for the movie.
        
        Returns:
        - dict: A dictionary containing information about the movie.
        """
        endpoint = f"v1/rest/movie?uid={uid}"
        return mr.make_request(self.base_url+endpoint)
        
    def get_movies(self, pageNumber=0, pageSize=50):
        """
        Returns a list of Star Trek movies of the specified page number and page size.
        
        Args:
        - pageNumber (int): The page number of the list. Defaults to 0, the first page.
        - pageSize (int): The number of movies per page. Defaults to 50 (although at this time there are only 14 entries).
        
        Returns:
        - dict: A dictionary containing a list of movies.
        """
        endpoint = f"v1/rest/movie/search?pageNumber={pageNumber}&pageSize={pageSize}"
        return mr.make_request(self.base_url+endpoint)
        
    def get_occupation_by_id(self, uid):
        """
        Returns information about a Star Trek occupation matching the unique id.
        
        Args:
        - uid (str): The unique identifier for the occupation.
        
        Returns:
        - dict: A dictionary containing information about the occupation.
        """
        endpoint = f"v2/rest/occupation?uid={uid}"
        return mr.make_request(self.base_url+endpoint)
        
    def get_occupations(self, pageNumber=0, pageSize=50):
        """
        Returns a list of Star Trek occupations of the specified page number and page size.
        
        Args:
        - pageNumber (int): The page number of the list. Defaults to 0, the first page.
        - pageSize (int): The number of occupations per page. Defaults to 50.
        
        Returns:
        - dict: A dictionary containing a list of occupations.
        """
        endpoint = f"v2/rest/occupation/search?pageNumber={pageNumber}&pageSize={pageSize}"
        return mr.make_request(self.base_url+endpoint)
        
    def get_organization_by_id(self, uid):
        """
        Returns information about a Star Trek organization matching the unique id.
        
        Args:
        - uid (str): The unique identifier for the organization.
        
        Returns:
        - dict: A dictionary containing information about the organization.
        """
        endpoint = f"v1/rest/organization?uid={uid}"
        return mr.make_request(self.base_url+endpoint)
        
    def get_organizations(self, pageNumber=0, pageSize=50):
        """
        Returns a list of Star Trek organizations of the specified page number and page size.
        
        Args:
        - pageNumber (int): The page number of the list. Defaults to 0, the first page.
        - pageSize (int): The number of organizations per page. Defaults to 50.
        
        Returns:
        - dict: A dictionary containing a list of organizations.
        """
        endpoint = f"v1/rest/organization/search?pageNumber={pageNumber}&pageSize={pageSize}"
        return mr.make_request(self.base_url+endpoint)
        
    def get_performer_by_id(self, uid):
        """
        Returns information about a Star Trek performer matching the unique id.
        
        Args:
        - uid (str): The unique identifier for the performer.
        
        Returns:
        - dict: A dictionary containing information about the performer.
        """
        endpoint = f"v2/rest/performer?uid={uid}"
        return mr.make_request(self.base_url+endpoint)
        
    def get_performers(self, pageNumber=0, pageSize=50):
        """
        Returns a list of Star Trek performers of the specified page number and page size.
        
        Args:
        - pageNumber (int): The page number of the list. Defaults to 0, the first page.
        - pageSize (int): The number of performers per page. Defaults to 50.
        
        Returns:
        - dict: A dictionary containing a list of performers.
        """
        endpoint = f"v2/rest/performer/search?pageNumber={pageNumber}&pageSize={pageSize}"
        return mr.make_request(self.base_url+endpoint)
        
    def get_season_by_id(self, uid):
        """
        Returns information about a Star Trek season matching the unique id.
        
        Args:
        - uid (str): The unique identifier for the season.
        
        Returns:
        - dict: A dictionary containing information about the season.
        """
        endpoint = f"v1/rest/season?uid={uid}"
        return mr.make_request(self.base_url+endpoint)
        
    def get_seasons(self, pageNumber=0, pageSize=50):
        """
        Returns a list of Star Trek seasons of the specified page number and page size.
        
        Args:
        - pageNumber (int): The page number of the list. Defaults to 0, the first page.
        - pageSize (int): The number of seasons per page. Defaults to 50.
        
        Returns:
        - dict: A dictionary containing a list of seasons.
        """
        endpoint = f"v1/rest/season/search?pageNumber={pageNumber}&pageSize={pageSize}"
        return mr.make_request(self.base_url+endpoint)
        
    def get_series_by_id(self, uid):
        """
        Returns information about a Star Trek series matching the unique id.
        
        Args:
        - uid (str): The unique identifier for the series.
        
        Returns:
        - dict: A dictionary containing information about the series.
        """
        endpoint = f"v1/rest/series?uid={uid}"
        return mr.make_request(self.base_url+endpoint)
        
    def get_series(self, pageNumber=0, pageSize=50):
        """
        Returns a list of Star Trek series of the specified page number and page size.
        
        Args:
        - pageNumber (int): The page number of the list. Defaults to 0, the first page.
        - pageSize (int): The number of series per page. Defaults to 50 (although at this time there are only 12 entries).
        
        Returns:
        - dict: A dictionary containing a list of series.
        """
        endpoint = f"v1/rest/series/search?pageNumber={pageNumber}&pageSize={pageSize}"
        return mr.make_request(self.base_url+endpoint)
        
    def get_soundtrack_by_id(self, uid):
        """
        Returns information about a Star Trek soundtrack.matching the unique id.
        
        Args:
        - uid (str): The unique identifier for the soundtrack.
        
        Returns:
        - dict: A dictionary containing information about the soundtrack.
        """
        endpoint = f"v1/rest/soundtrack?uid={uid}"
        return mr.make_request(self.base_url+endpoint)
        
    def get_soundtracks(self, pageNumber=0, pageSize=50):
        """
        Returns a list of Star Trek soundtracks of the specified page number and page size.
        
        Args:
        - pageNumber (int): The page number of the list. Defaults to 0, the first page.
        - pageSize (int): The number of soundtracks per page. Defaults to 50 (although at this time there are only 12 entries).
        
        Returns:
        - dict: A dictionary containing a list of soundtracks.
        """
        endpoint = f"v1/rest/soundtrack/search?pageNumber={pageNumber}&pageSize={pageSize}"
        return mr.make_request(self.base_url+endpoint)
        
    def get_spacecraft_by_id(self, uid):
        """
        Returns information about a Star Trek spacecraft matching the unique id.
        
        Args:
        - uid (str): The unique identifier for the spacecraft.
        
        Returns:
        - dict: A dictionary containing information about the spacecraft.
        """
        endpoint = f"v2/rest/spacecraft?uid={uid}"
        return mr.make_request(self.base_url+endpoint)
        
    def get_spacecrafts(self, pageNumber=0, pageSize=50):
        """
        Returns a list of Star Trek spacecrafts of the specified page number and page size.
        
        Args:
        - pageNumber (int): The page number of the list. Defaults to 0, the first page.
        - pageSize (int): The number of spacecrafts per page. Defaults to 50.
        
        Returns:
        - dict: A dictionary containing a list of spacecrafts.
        """
        endpoint = f"v2/rest/spacecraft/search?pageNumber={pageNumber}&pageSize={pageSize}"
        return mr.make_request(self.base_url+endpoint)
        
    def get_spacecraft_class_by_id(self, uid):
        """
        Returns information about a Star Trek spacecraft class matching the unique id.
        
        Args:
        - uid (str): The unique identifier for the spacecraft class.
        
        Returns:
        - dict: A dictionary containing information about the spacecraft class.
        """
        endpoint = f"v2/rest/spacecraftClass?uid={uid}"
        return mr.make_request(self.base_url+endpoint)
        
    def get_spacecraft_classes(self, pageNumber=0, pageSize=50):
        """
        Returns a list of Star Trek spacecraft classes of the specified page number and page size.
        
        Args:
        - pageNumber (int): The page number of the list. Defaults to 0, the first page.
        - pageSize (int): The number of spacecraft classes per page. Defaults to 50.
        
        Returns:
        - dict: A dictionary containing a list of spacecraft classes.
        """
        endpoint = f"v2/rest/spacecraftClass/search?pageNumber={pageNumber}&pageSize={pageSize}"
        return mr.make_request(self.base_url+endpoint)
        
    def get_species_by_id(self, uid):
        """
        Returns information about a Star Trek species matching the unique id.
        
        Args:
        - uid (str): The unique identifier for the species.
        
        Returns:
        - dict: A dictionary containing information about the species.
        """
        endpoint = f"v2/rest/species?uid={uid}"
        return mr.make_request(self.base_url+endpoint)
        
    def get_species(self, pageNumber=0, pageSize=50):
        """
        Returns a list of Star Trek species of the specified page number and page size.
        
        Args:
        - pageNumber (int): The page number of the list. Defaults to 0, the first page.
        - pageSize (int): The number of species per page. Defaults to 50.
        
        Returns:
        - dict: A dictionary containing a list of species.
        """
        endpoint = f"v2/rest/species/search?pageNumber={pageNumber}&pageSize={pageSize}"
        return mr.make_request(self.base_url+endpoint)
        
    def get_staff_by_id(self, uid):
        """
        Returns information about a Star Trek staff matching the unique id.
        
        Args:
        - uid (str): The unique identifier for the staff.
        
        Returns:
        - dict: A dictionary containing information about the staff.
        """
        endpoint = f"v2/rest/staff?uid={uid}"
        return mr.make_request(self.base_url+endpoint)
        
    def get_staff(self, pageNumber=0, pageSize=50):
        """
        Returns a list of Star Trek staff of the specified page number and page size.
        
        Args:
        - pageNumber (int): The page number of the list. Defaults to 0, the first page.
        - pageSize (int): The number of staff per page. Defaults to 50.
        
        Returns:
        - dict: A dictionary containing a list of staff.
        """
        endpoint = f"v2/rest/staff/search?pageNumber={pageNumber}&pageSize={pageSize}"
        return mr.make_request(self.base_url+endpoint)
        
    def get_technology_by_id(self, uid):
        """
        Returns information about a Star Trek technology matching the unique id.
        
        Args:
        - uid (str): The unique identifier for the technology.
        
        Returns:
        - dict: A dictionary containing information about the technology.
        """
        endpoint = f"v2/rest/technology?uid={uid}"
        return mr.make_request(self.base_url+endpoint)
        
    def get_technologies(self, pageNumber=0, pageSize=50):
        """
        Returns a list of Star Trek technologies of the specified page number and page size.
        
        Args:
        - pageNumber (int): The page number of the list. Defaults to 0, the first page.
        - pageSize (int): The number of technologies per page. Defaults to 50.
        
        Returns:
        - dict: A dictionary containing a list of technologies.
        """
        endpoint = f"v2/rest/technology/search?pageNumber={pageNumber}&pageSize={pageSize}"
        return mr.make_request(self.base_url+endpoint)
        
    def get_title_by_id(self, uid):
        """
        Returns information about a Star Trek titles matching the unique id.
        
        Args:
        - uid (str): The unique identifier for the title.
        
        Returns:
        - dict: A dictionary containing information about the title.
        """
        endpoint = f"v2/rest/title?uid={uid}"
        return mr.make_request(self.base_url+endpoint)
        
    def get_titles(self, pageNumber=0, pageSize=50):
        """
        Returns a list of Star Trek titles of the specified page number and page size.
        
        Args:
        - pageNumber (int): The page number of the list. Defaults to 0, the first page.
        - pageSize (int): The number of titles per page. Defaults to 50.
        
        Returns:
        - dict: A dictionary containing a list of titles.
        """
        endpoint = f"v2/rest/title/search?pageNumber={pageNumber}&pageSize={pageSize}"
        return mr.make_request(self.base_url+endpoint)
        
    def get_trading_card_by_id(self, uid):
        """
        Returns information about a Star Trek trading card matching the unique id.
        
        Args:
        - uid (str): The unique identifier for the trading card.
        
        Returns:
        - dict: A dictionary containing information about the trading card.
        """
        endpoint = f"v1/rest/tradingCard?uid={uid}"
        return mr.make_request(self.base_url+endpoint)
        
    def get_trading_cards(self, pageNumber=0, pageSize=50):
        """
        Returns a list of Star Trek trading cards of the specified page number and page size.
        
        Args:
        - pageNumber (int): The page number of the list. Defaults to 0, the first page.
        - pageSize (int): The number of trading cards per page. Defaults to 50.
        
        Returns:
        - dict: A dictionary containing a list of trading cards.
        """
        endpoint = f"v1/rest/tradingCard/search?pageNumber={pageNumber}&pageSize={pageSize}"
        return mr.make_request(self.base_url+endpoint)
        
    def get_trading_card_deck_by_id(self, uid):
        """
        Returns information about a Star Trek trading card deck matching the unique id.
        
        Args:
        - uid (str): The unique identifier for the trading card deck.
        
        Returns:
        - dict: A dictionary containing information about the trading card deck.
        """
        endpoint = f"v1/rest/tradingCardDeck?uid={uid}"
        return mr.make_request(self.base_url+endpoint)
        
    def get_trading_card_decks(self, pageNumber=0, pageSize=50):
        """
        Returns a list of Star Trek trading card decks of the specified page number and page size.
        
        Args:
        - pageNumber (int): The page number of the list. Defaults to 0, the first page.
        - pageSize (int): The number of trading card decks per page. Defaults to 50.
        
        Returns:
        - dict: A dictionary containing a list of trading card decks.
        """
        endpoint = f"v1/rest/tradingCardDeck/search?pageNumber={pageNumber}&pageSize={pageSize}"
        return mr.make_request(self.base_url+endpoint)
        
    def get_trading_card_set_by_id(self, uid):
        """
        Returns information about a Star Trek trading card set matching the unique id.
        
        Args:
        - uid (str): The unique identifier for the trading card set.
        
        Returns:
        - dict: A dictionary containing information about the trading card set.
        """
        endpoint = f"v1/rest/tradingCardSet?uid={uid}"
        return mr.make_request(self.base_url+endpoint)
        
    def get_trading_card_sets(self, pageNumber=0, pageSize=50):
        """
        Returns a list of Star Trek trading card sets of the specified page number and page size.
        
        Args:
        - pageNumber (int): The page number of the list. Defaults to 0, the first page.
        - pageSize (int): The number of trading card sets per page. Defaults to 50.
        
        Returns:
        - dict: A dictionary containing a list of trading card sets.
        """
        endpoint = f"v1/rest/tradingCardSet/search?pageNumber={pageNumber}&pageSize={pageSize}"
        return mr.make_request(self.base_url+endpoint)
        
    def get_video_game_by_id(self, uid):
        """
        Returns information about a Star Trek video game matching the unique id.
        
        Args:
        - uid (str): The unique identifier for the video game.
        
        Returns:
        - dict: A dictionary containing information about the video game.
        """
        endpoint = f"v1/rest/videoGame?uid={uid}"
        return mr.make_request(self.base_url+endpoint)
        
    def get_video_games(self, pageNumber=0, pageSize=50):
        """
        Returns a list of Star Trek video games of the specified page number and page size.
        
        Args:
        - pageNumber (int): The page number of the list. Defaults to 0, the first page.
        - pageSize (int): The number of video games per page. Defaults to 50.
        
        Returns:
        - dict: A dictionary containing a list of video games.
        """
        endpoint = f"v1/rest/videoGame/search?pageNumber={pageNumber}&pageSize={pageSize}"
        return mr.make_request(self.base_url+endpoint)
        
    def get_video_release_by_id(self, uid):
        """
        Returns information about a Star Trek video release matching the unique id.
        
        Args:
        - uid (str): The unique identifier for the video release.
        
        Returns:
        - dict: A dictionary containing information about the video release.
        """
        endpoint = f"v2/rest/videoRelease?uid={uid}"
        return mr.make_request(self.base_url+endpoint)
        
    def get_video_releases(self, pageNumber=0, pageSize=50):
        """
        Returns a list of Star Trek video releases of the specified page number and page size.
        
        Args:
        - pageNumber (int): The page number of the list. Defaults to 0, the first page.
        - pageSize (int): The number of video releases per page. Defaults to 50.
        
        Returns:
        - dict: A dictionary containing a list of video releases.
        """
        endpoint = f"v2/rest/videoRelease/search?pageNumber={pageNumber}&pageSize={pageSize}"
        return mr.make_request(self.base_url+endpoint)
        
    def get_weapon_by_id(self, uid):
        """
        Returns information about a Star Trek weapon matching the unique id.
        
        Args:
        - uid (str): The unique identifier for the weapon.
        
        Returns:
        - dict: A dictionary containing information about the weapon.
        """
        endpoint = f"v2/rest/weapon?uid={uid}"
        return mr.make_request(self.base_url+endpoint)
        
    def get_weapons(self, pageNumber=0, pageSize=50):
        """
        Returns a list of Star Trek weapons of the specified page number and page size.
        
        Args:
        - pageNumber (int): The page number of the list. Defaults to 0, the first page.
        - pageSize (int): The number of weapons per page. Defaults to 50.
        
        Returns:
        - dict: A dictionary containing a list of weapons.
        """
        endpoint = f"v2/rest/weapon/search?pageNumber={pageNumber}&pageSize={pageSize}"
        return mr.make_request(self.base_url+endpoint)
       
        
        
    
       
       
       
       
       
       
       
       
       
       
     
     
     
        
    
    
 
 
 
   
   
   
   
   
   
     
   
   
 
 
 
   
  
   
 
  
   
   
  
    
   
  
  
   
  
   
        


from .. helperFuncs import make_request as mr


class IP_API:
    """
    A class to interact with the IP API.
    
    Attributes:
    - base_url: The base URL of IP API.
    - about: A short description of the API.
    """
    def __init__(self):
        self.base_url = "http://ip-api.com/json/" 
        self.about = "The IP API is a fast, reliable, and free IP geolocation API."
        
    def get_docs_url(self):
        """
        Returns URL for API docs.
        
        Args:
        - None
        
        Returns:
        - string: URL for API documentation.
        """
        return "https://ip-api.com/docs"
    
    def get_location_info_by_ip_address(self, ip):
        """
        Returns geolocation info (country, state, coordinates, etc.) by IP address.

        Args:
        - ip (str): IPv4/IPv6 address or domain name.

        Returns:
        - dict: Location information based on given IP address or domain name.
        """
        endpoint = f"{ip}"
        return mr.make_request(self.base_url+endpoint)
    